;=====================================================================
;
; File        lib-varh.ASM
;
; Purpose     To provide a gneral set of routines to handle access to
;             variables indexed by Registers IX and IY.
;
; Copyright (c) 2010 Ray Palmer
;
; Created by  R. Palmer
; Created on  15 Nov 2013
; Updated on  15 Nov 2013
;
; Notes       1. Z80 is little-Endian format when direct save is
;                used/referenced (e.g LD (xxxx),HL)
;
;=====================================================================
;
;   Routines to get variables 16/32 bits (Big-Endian)
;
;=====================================================================
;Name     VBE_GetValBC
;Purpose  To get the value of the variable (16bit)
;Input    HL = address of 16bit variable.
;Output   BC = value of the variable.
;         HL = HL + 2
;Notes    1. Numer is in Big-Endian format.
;=====================================================================
VBE_GetValBC
   LD    B,(HL)
   INC   HL
   LD    C,(HL)
   INC   HL
   RET
;=====================================================================
;Name     VBE_GetValDE
;Purpose  To get the value of the variable (16bit)
;Input    HL = address of 16bit variable.
;Output   DE = value of the variable.
;         HL = HL + 2
;Notes    1. Numer is in Big-Endian format.
;=====================================================================
VBE_GetValDE
   LD    D,(HL)
   INC   HL
   LD    E,(HL)
   INC   HL
   RET
;=====================================================================
;Name     VBE_GetValHL
;Purpose  To get the value of the variable (16bit)
;Input    HL = address of 16bit variable.
;Output   HL = value of the variable.
;Notes    1. Numer is in Big-Endian format.
;=====================================================================
VBE_GetValHL
   PUSH  AF
   LD    A,(HL)
   INC   HL
   LD    L,(HL)
   LD    H,A
   POP   AF
   RET
;=====================================================================
;Name     VBE_GetVal32
;Purpose  To get the value of the variable (32bit)
;Input    HL = address of 16bit variable.
;Output   DE/BC = value of the variable.
;         HL = address of 16bit variable.
;Notes    1. Numer is in Big-Endian format.
;=====================================================================
VBE_GetVal32
   LD    D,(HL)
   INC   HL
   LD    E,(HL)
   INC   HL
   LD    B,(HL)
   INC   HL
   LD    C,(HL)
   INC   HL
   RET
;=====================================================================
;=====================================================================
;
;   Routines to set variables 16/32 bits (Big-Endian)
;
;=====================================================================
;Name     VBE_SetValBC
;Purpose  To get the value of the variable (16bit)
;Input    BC = value of the variable.
;         HL = address of 16bit variable.
;Output   HL = address of 16bit variable + 2
;Notes    1. Numer is saved in Big-Endian format.
;=====================================================================
VBE_SetValBC
   LD    (HL),B
   INC   HL
   LD    (HL),C
   INC   HL
   RET
;=====================================================================
;Name     VBE_SetValDE
;Purpose  To set the value of the variable (16bit)
;Input    HL = address of 16bit variable.
;         DE = value to set
;Output   HL = HL + 2
;Notes    1. Numer is saved in big-Endian format.
;=====================================================================
VBE_SetValDE
   LD    (HL),D
   INC   HL
   LD    (HL),E
   INC   HL
   RET
;=====================================================================
;Name     VBE_SetVal32
;Purpose  To set the value of the variable (16bit)
;Input    HL = address of 32bit variable.
;         DE/BC = value to set
;Output   HL = HL + 2
;Notes    1. Numer is saved in big-Endian format.
;=====================================================================
VBE_SetVal32
   LD    (HL),D
   INC   HL
   LD    (HL),E
   INC   HL
   LD    (HL),B
   INC   HL
   LD    (HL),C
   INC   HL
   RET
;=====================================================================
;=====================================================================
;
;   Routines to get variables 16/32 bits (Little-Endian)
;
;=====================================================================
;Name     VLE_GetValBC
;Purpose  To get the value of the variable (16bit)
;Input    HL = address of 16bit variable.
;Output   BC = value of the variable.
;         HL = HL + 2
;Notes    1. Numer is in Big-Endian format.
;=====================================================================
VLE_GetValBC
   LD    C,(HL)
   INC   HL
   LD    B,(HL)
   INC   HL
   RET
;=====================================================================
;Name     VLE_GetValDE
;Purpose  To get the value of the variable (16bit)
;Input    HL = address of 16bit variable.
;Output   DE = value of the variable.
;         HL = HL + 2
;Notes    1. Numer is in Big-Endian format.
;=====================================================================
VLE_GetValDE
   LD    E,(HL)
   INC   HL
   LD    D,(HL)
   INC   HL
   RET
;=====================================================================
;Name     VLE_GetValHL
;Purpose  To get the value of the variable (16bit)
;Input    HL = address of 16bit variable.
;Output   HL = value of the variable.
;Notes    1. Numer is in Big-Endian format.
;=====================================================================
VLE_GetValHL
   PUSH  AF
   LD    A,(HL)
   INC   HL
   LD    H,(HL)
   LD    L,A
   POP   AF
   RET
;=====================================================================
;Name     VLE_GetVal32
;Purpose  To get the value of the variable (32bit)
;Input    HL = address of 16bit variable.
;Output   DE/BC = value of the variable.
;         HL = address of 16bit variable.
;Notes    1. Numer is in Big-Endian format.
;=====================================================================
VLE_GetVal32
   LD    C,(HL)
   INC   HL
   LD    B,(HL)
   INC   HL
   LD    E,(HL)
   INC   HL
   LD    D,(HL)
   INC   HL
   RET
;=====================================================================
;=====================================================================
;
;   Routines to set variables 16/32 bits (Little-Endian)
;
;=====================================================================
;Name     VLE_SetValBC
;Purpose  To get the value of the variable (16bit)
;Input    BC = value of the variable.
;         HL = address of 16bit variable.
;Output   HL = address of 16bit variable + 2
;Notes    1. Numer is saved in Big-Endian format.
;=====================================================================
VLE_SetValBC
   LD    (HL),C
   INC   HL
   LD    (HL),B
   INC   HL
   RET
;=====================================================================
;Name     VLE_SetValDE
;Purpose  To set the value of the variable (16bit)
;Input    HL = address of 16bit variable.
;         DE = value to set
;Output   HL = HL + 2
;Notes    1. Numer is saved in big-Endian format.
;=====================================================================
VLE_SetValDE
   LD    (HL),E
   INC   HL
   LD    (HL),D
   INC   HL
   RET
;=====================================================================
;Name     VLE_SetVal32
;Purpose  To set the value of the variable (16bit)
;Input    HL = address of 32bit variable.
;         DE/BC = value to set
;Output   HL = HL + 2
;Notes    1. Numer is saved in big-Endian format.
;=====================================================================
VLE_SetVal32
   LD    (HL),C
   INC   HL
   LD    (HL),B
   INC   HL
   LD    (HL),E
   INC   HL
   LD    (HL),D
   INC   HL
   RET
;=====================================================================
;=====================================================================
;
;   Routines to access variables 8/16/32 bits via Reg IY
;
;=====================================================================
;Name     VIY_GetVar8
;Purpose  To get the value of a variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;Output   HL = address of variable 
;         A = Value of the variable
;Notes    None.
;=====================================================================
VIY_GetVar8
   CALL  VIY_CalcAddr
   LD    A,(HL)
   RET
;=====================================================================
;Name     VIYBE_GetVarBC
;Purpose  To get the value of a Big-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;Output   HL = address of variable 
;         BC = Value of the variable
;Notes    None.
;=====================================================================
VIYBE_GetVarBC
   CALL  VIY_CalcAddr
   JP    VBE_GetValBC
;=====================================================================
;Name     VIYBE_GetVarDE
;Purpose  To get the value of a Big-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;Output   HL = address of variable 
;         DE = Value of the variable
;Notes    None.
;=====================================================================
VIYBE_GetVarDE
   CALL  VIY_CalcAddr
   JP    VBE_GetValDE
;=====================================================================
;Name     VIYBE_GetVarHL
;Purpose  To get the value of a Big-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;Output   HL = Value got
;Notes    None.
;=====================================================================
VIYBE_GetVarHL
   CALL  VIY_CalcAddr
   JP    VBE_GetValHL
;=====================================================================
;Name     VIYBE_GetVar32
;Purpose  To get the value of a Big-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;Output   HL = address of variable 
;         BC/DE = Value of the variable
;Notes    None.
;=====================================================================
VIYBE_GetVar32
   CALL  VIY_CalcAddr
   JP    VBE_GetVal32
;=====================================================================
;Name     VIYLE_GetVarBC
;Purpose  To get the value of a Little-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;Output   HL = address of variable 
;         BC = Value of the variable
;Notes    None.
;=====================================================================
VIYLE_GetVarBC
   CALL  VIY_CalcAddr
   JP    VLE_GetValBC
;=====================================================================
;Name     VIYLE_GetVarDE
;Purpose  To get the value of a Little-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;Output   HL = address of variable 
;         DE = Value of the variable
;Notes    None.
;=====================================================================
VIYLE_GetVarDE
   CALL  VIY_CalcAddr
   JP    VLE_GetValDE
;=====================================================================
;Name     VIYLE_GetVarHL
;Purpose  To get the value of a Little-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;Output   HL = Value got
;Notes    None.
;=====================================================================
VIYLE_GetVarHL
   CALL  VIY_CalcAddr
   JP    VLE_GetValHL
;=====================================================================
;Name     VIYBE_GetVar32
;Purpose  To get the value of a Little-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;Output   HL = address of variable 
;         BC/DE = Value of the variable
;Notes    None.
;=====================================================================
VIYBE_GetVar32
   CALL  VIY_CalcAddr
   JP    VBE_GetVal32
;=====================================================================
;Name     VIY_SetVar8
;Purpose  To set the value of a variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;Output   HL = address of variable 
;         A = Value of the variable
;Notes    None.
;=====================================================================
VIY_SetVar8
   CALL  VIY_CalcAddr
   LD    (HL),A
   RET
;=====================================================================
;Name     VIYBE_SetVarBC
;Purpose  To set the value of a Big-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;         BC = Value of the variable
;Output   HL = address of variable + 2
;Notes    None.
;=====================================================================
VIYBE_SetVarBC
   CALL  VIY_CalcAddr
   JP    VBE_SetValBC
;=====================================================================
;Name     VIYBE_SetVarDE
;Purpose  To set the value of a Big-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;         DE = Value of the variable
;Output   HL = address of variable + 2
;Notes    None.
;=====================================================================
VIYBE_SetVarDE
   CALL  VIY_CalcAddr
   JP    VBE_SetValDE
;=====================================================================
;Name     VIYBE_SetVar32
;Purpose  To set the value of a Big-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;         BC/DE = Value of the variable
;Output   HL = address of variable + 4
;Notes    None.
;=====================================================================
VIYBE_SetVar32
   CALL  VIY_CalcAddr
   JP    VBE_SetVal32
;=====================================================================
;Name     VIYLE_SetVarBC
;Purpose  To set the value of a Little-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;         BC = Value of the variable
;Output   HL = address of variable + 2
;Notes    None.
;=====================================================================
VIYLE_SetVarBC
   CALL  VIY_CalcAddr
   JP    VLE_SetValBC
;=====================================================================
;Name     VIYLE_SetVarDE
;Purpose  To set the value of a Little-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;         DE = Value of the variable
;Output   HL = address of variable + 2
;Notes    None.
;=====================================================================
VIYLE_SetVarDE
   CALL  VIY_CalcAddr
   JP    VLE_SetValDE
;=====================================================================
;Name     VIYLE_SetVar32
;Purpose  To Set the value of a Little-Endian variable using IY.
;Input    IY = address of variables
;         HL = offset to variable
;         BC/DE = Value of the variable
;Output   HL = address of variable + 4
;Notes    None.
;=====================================================================
VIYLE_SetVar32
   CALL  VIY_CalcAddr
   JP    VLE_LetVal32
;=====================================================================
;=====================================================================
;
;   Routines to access variables 8/16/32 bits via Reg IX
;
;=====================================================================
;Name     VIX_GetVar8
;Purpose  To get the value of a variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;Output   HL = address of variable 
;         A = Value of the variable
;Notes    None.
;=====================================================================
VIX_GetVar8
   CALL  VIX_CalcAddr
   LD    A,(HL)
   RET
;=====================================================================
;Name     VIXBE_GetVarBC
;Purpose  To get the value of a Big-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;Output   HL = address of variable 
;         BC = Value of the variable
;Notes    None.
;=====================================================================
VIXBE_GetVarBC
   CALL  VIX_CalcAddr
   JP    VBE_GetValBC
;=====================================================================
;Name     VIXBE_GetVarDE
;Purpose  To get the value of a Big-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;Output   HL = address of variable 
;         DE = Value of the variable
;Notes    None.
;=====================================================================
VIXBE_GetVarDE
   CALL  VIX_CalcAddr
   JP    VBE_GetValDE
;=====================================================================
;Name     VIXBE_GetVarHL
;Purpose  To get the value of a Big-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;Output   HL = Value got
;Notes    None.
;=====================================================================
VIXBE_GetVarHL
   CALL  VIX_CalcAddr
   JP    VBE_GetValHL
;=====================================================================
;Name     VIXBE_GetVar32
;Purpose  To get the value of a Big-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;Output   HL = address of variable + 4
;         BC/DE = Value of the variable
;Notes    None.
;=====================================================================
VIXBE_GetVar32
   CALL  VIX_CalcAddr
   JP    VBE_GetVal32
;=====================================================================
;Name     VIXLE_GetVarBC
;Purpose  To get the value of a Little-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;Output   HL = address of variable + 2
;         BC = Value of the variable
;Notes    None.
;=====================================================================
VIXLE_GetVarBC
   CALL  VIX_CalcAddr
   JP    VLE_GetValBC
;=====================================================================
;Name     VIXLE_GetVarDE
;Purpose  To get the value of a Little-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;Output   HL = address of variable + 2
;         DE = Value of the variable
;Notes    None.
;=====================================================================
VIXLE_GetVarDE
   CALL  VIX_CalcAddr
   JP    VLE_GetValDE
;=====================================================================
;Name     VIXLE_GetVarHL
;Purpose  To get the value of a Little-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;Output   HL = Value got
;Notes    None.
;=====================================================================
VIXLE_GetVarHL
   CALL  VIX_CalcAddr
   JP    VLE_GetValHL
;=====================================================================
;Name     VIXLE_GetVar32
;Purpose  To get the value of a Little-Endian variable using IX.
;Input    HL = offset to variable
;Output   HL = address of variable 
;         BC/DE = Value of the variable
;Notes    None.
;=====================================================================
VIXLE_GetVar32
   CALL  VIX_CalcAddr
   JP    VLE_GetVal32
;=====================================================================
;Name     VIX_SetVar8
;Purpose  To set the value of a variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;Output   HL = address of variable
;         A = Value of the variable
;Notes    None.
;=====================================================================
VIX_SetVar8
   CALL  VIX_CalcAddr
   LD    (HL),A
   RET
;=====================================================================
;Name     VIXBE_SetVarBC
;Purpose  To set the value of a Big-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;         BC = Value of the variable
;Output   HL = address of variable + 2
;Notes    None.
;=====================================================================
VIXBE_SetVarBC
   CALL  VIX_CalcAddr
   JP    VBE_SetValBC
;=====================================================================
;Name     VIXBE_SetVarDE
;Purpose  To set the value of a Big-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;         DE = Value of the variable
;Output   HL = address of variable + 2
;Notes    None.
;=====================================================================
VIXBE_SetVarDE
   CALL  VIX_CalcAddr
   JP    VBE_SetValDE
;=====================================================================
;Name     VIXBE_SetVar32
;Purpose  To set the value of a Big-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;         BC/DE = Value of the variable
;Output   HL = address of variable + 4
;Notes    None.
;=====================================================================
VIXBE_SetVar32
   CALL  VIX_CalcAddr
   JP    VBE_SetVal32
;=====================================================================
;Name     VIXLE_SetVarBC
;Purpose  To set the value of a Little-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;         BC = Value of the variable
;Output   HL = address of variable + 2
;Notes    None.
;=====================================================================
VIXLE_SetVarBC
   CALL  VIX_CalcAddr
   JP    VLE_SetValBC
;=====================================================================
;Name     VIXLE_SetVarDE
;Purpose  To set the value of a Little-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;         DE = Value of the variable
;Output   HL = address of variable + 2
;Notes    None.
;=====================================================================
VIXLE_SetVarDE
   CALL  VIX_CalcAddr
   JP    VLE_SetValDE
;=====================================================================
;Name     VIXLE_SetVar32
;Purpose  To Set the value of a Little-Endian variable using IX.
;Input    IX = address of variables
;         HL = offset to variable
;         BC/DE = Value of the variable
;Output   HL = address of variable + 4
;Notes    None.
;=====================================================================
VIXLE_SetVar32
   CALL  VIX_CalcAddr
   JP    VLE_LetVal32
;