;=====================================================================
;
; TCP/IP structure definitions and contants
; Copyright (c) 2010 Ray Palmer
;
; All data in the structures below are defined as Big Endian, that is
; they are stored in with the MSB first followed by the LSB.  The data
; payload (i.e Application Data) may be stored in the hosts native
; format.
;
; Structure of the TCP/IP Communication packet is
;
;                                 Layer
;  +---------------------------+
;  | Device Header             |  Hardware (e.g ethernet)
;  +---------------------------+
;  |  IP Header Data           |  Internet Protocol
;  +---------------------------+
;  |   TCP / UDP / ICMP        |  TCP / UDP / ICMP Header
;  +---------------------------+
;  |    (TCP / UDP / App) Data |  Application Data + Header (if any)
;  +---------------------------+
;  | Device Trailer - if any   |  Hardware (e.g ethernet)
;  +---------------------------+
;
; At the Application Layer of data there maybe a breakdown of the data
; format which may include the application specific header along
; with the application data.
;
; Example applications using TCP/IP are -
;  1.  File Transfer Protocol (FTP)
;  2.  HyperText Protocol (HTTP)
;  3.  Simple Mail Transfer Protocol (SMTP)
;  4.  Domain Name Server Protocol (DNS)
;  5.  Telnet
;  6.  HDOS (New)
;
; The hardware layer is controlled by 
;  1. Logical Link Controller   (LLC - Ethernet)
;
;=====================================================================
;  Structure of IPv4 header data packet with addtional options
;  - Values store in structure are in Big-Endian Format
;  - All data is stored in big-endian format (i.e MSB then LSB)
;  - The IP_PKT_DATA label should be calculated from IP_PKT_HDRLEN
;    and IP_PKT_OPTIONS
;  - The Min IP Header Length is 5 x 32 bits (or 20 bytes).
;  - Source of information URL
;    http //en.wikipedia.org/wiki/IPv4
;  - Min IP header size is at IP_PKT_OPTIONS
;=====================================================================
IP_PKT_VERS       	EQU  0  ;(1) Lower nibble - Value 4 = IPv4
IP_PKT_HDRLEN     	EQU  0  ;(1) Upper nibble - In 32bits (5 to 15)
IP_PKT_DIFFSERV   	EQU  1  ;(1) Differentiated Services
IP_PKT_TOTLEN     	EQU  2  ;(2) Total length in Bytes
IP_PKT_IDENT      	EQU  4  ;(2) Identification
IP_PKT_FLGS       	EQU  6  ;(1) Lower 3 bits
IP_PKT_FRAGOFF    	EQU  6  ;(2) 13 bits Fragment offset
IP_PKT_TTL        	EQU  8  ;(1) Time To Live (TTL)
IP_PKT_PRTCOL     	EQU  9  ;(1) Protocol
IP_PKT_HDRCKSUM   	EQU  10 ;(2) Header Checksum
IP_PKT_SRCADR     	EQU  12 ;(4) IPv4 Source Address
IP_PKT_DSTADR     	EQU  16 ;(4) IPv4 Destination Address
IP_PKT_OPTIONS    	EQU  20 ;(N) Options Data (Max 10 x 32bits)
IP_PKT_DATA       	EQU  20 ;(M) Data of TCP/IP Packet
;
IP_PKT_HDRSIZE    	EQU  IP_PKT_DATA
;=====================================================================
;  Structure of TCP data packet with addtional options
;  - Values store in structure are in Big-Endian Format
;  - The TCP_PKT_DATA label should be calculated from TCP_PKT_HDRLEN
;    and TCP_PKT_OPTIONS
;  - Source of Information URL
;    http //en.wikipedia.org/wiki/Transmission_Control_Protocol
;
;  TCP_PKT_FLAGS = Bit field of Pkt flags
;    Bit  Type   Description
;     0          Reserved
;     1          Reserved
;     2   URG    Urgent Pointer Flag
;     3   ACK    Acknowledge Flag
;     4   PSH    PUSH Function
;     5   RST    Reset Flag
;     6   SYN    Synchronize Function
;     7   FIN    Finish Flag
;
;  - Min TCP header size is at TCP_PKT_OPTIONS
;=====================================================================
TCP_PKT_SRCPORT   	EQU  0  ;(2) Source Port
TCP_PKT_DSTPORT   	EQU  2  ;(2) Destination Port
TCP_PKT_SEQNUM    	EQU  4  ;(4) Sequence Number
TCP_PKT_ACKNUM    	EQU  8  ;(4) Acknowledge Number
TCP_PKT_HDRSZ     	EQU  12 ;(1) Upper Nibble - In octets
TCP_PKT_FLAGS     	EQU  13 ;(1) TCP Flags
TCP_PKT_WNDSZ     	EQU  14 ;(2) Window Size
TCP_PKT_CHKSUM    	EQU  16 ;(2) Checksum
TCP_PKT_URGPTR    	EQU  18 ;(2) Urgent Pointer
TCP_PKT_OPTIONS   	EQU  20 ;(N) Options data (Max 10 x 32 bits)
TCP_PKT_DATA      	EQU  20 ;(M) Data
TCP_PKT_HDRSIZE   	EQU  TCP_PKT_DATA
;=====================================================================
; Structure of the UDP data packet
;  - Values store in structure are in Little-Endian Format
;  - Source of Information URL
;    http //en.wikipedia.org/wiki/User_Datagram_Protocol
;=====================================================================
UDP_PKT_SRCPORT   	EQU  0 ;(2) Source Port
UDP_PKT_DSTPORT   	EQU  2 ;(2) Destination Port
UDP_PKT_LENGTH    	EQU  4 ;(2) Length of the UDP Header + Data
UDP_PKT_CHKSUM    	EQU  6 ;(2) Checksum of UDP Data Packet
UDP_PKT_DATA      	EQU  8 ;(N) Data for UDP Packet
UDP_PKT_HDRSIZE   	EQU  8 ;(2) Size of Header (also Min size of UDP)
;=====================================================================
; Structure of the ICMP data packet
;  - Values store in structure are in Little-Endian Format
;  - Source of Information URL
;    http //en.wikipedia.org/wiki/Internet_Control_Message_Protocol
;  - This structure occurs in the Data area of the IP message
;    see label IP_PKT_DATA
;=====================================================================
ICMP_PKT_TYPE      	EQU  0 ;(1) Control Message Type
ICMP_PKT_CODE      	EQU  1 ;(1) 
ICMP_PKT_CHKSUM    	EQU  2 ;(2) Checksum of ICMP Data Packet
ICMP_PKT_ID        	EQU  4 ;(2) ID
ICMP_PKT_SEQ       	EQU  6 ;(2) Sequence Number
ICMP_PKT_HDRSIZE   	EQU  ICMP_PKT_SEQ+2
ICMP_PKT_DATA      	equ  ICMP_PKT_HDRSIZE
ICMP_PKT_DATAEND   	equ  ICMP_PKT_DATA+8
;=====================================================================
; Structure of the TFTP data packet
;  - Values store in structure are in Big-Endian Format
;  - Source of Information URL
;    http //en.wikipedia.org/wiki/Trivial_File_Transfer_Protocol
;  - This structure occurs in the Data area of the IP message
;    see label UDP_PKT_DATA
;=====================================================================
TFTPD_PKT_OPC       	equ  UDP_PKT_HDRSIZE	;TFTP Op Code
TFTPD_PKT_BLK      	equ  TFTPD_PKT_OPC+2	;TFTP Block Number
TFTPD_PKT_DATA		EQU  TFTPD_PKT_BLK+2	;TFTP File Data
TFTPD_PKT_ERC      	equ  TFTPD_PKT_OPC+2	;TFTP Error Code
TFTPD_PKT_ERRMSG		EQU  TFTPD_PKT_ERC+2	;TFTP Error Message
TFTPD_PKT_FILENAME	EQU  TFTPD_PKT_OPC+2	;TFTP FileName
;=====================================================================
; Structure of the DNS data packet
;  - Values store in structure are in Big-Endian Format
;  - Source of Information URL (unknown)
;  - This structure occurs in the Data area of the IP message
;    see label UDP_PKT_DATA
;=====================================================================
DNSD_ID			equ	UDP_PKT_HDRSIZE	 ;DNS ID offset
DNSD_QFLAGS			equ	UDP_PKT_HDRSIZE+2	 ;DNS Qry flags off
DNSD_RFLAGS			equ	UDP_PKT_HDRSIZE+3	 ;DNS Resp flags off
DNSD_QDCOUNT		equ	UDP_PKT_HDRSIZE+4	 ;16 bit query ctr
DNSD_ANCOUNT		equ	UDP_PKT_HDRSIZE+6	 ;16 bit Answer ctr
DNSD_NSCOUNT		equ	UDP_PKT_HDRSIZE+8	 ;16 bit NS ctr
DNSD_ARCOUNT		equ	UDP_PKT_HDRSIZE+10 ;16 bit auth Resp Ctr
DNSD_QNAME			equ	UDP_PKT_HDRSIZE+12 ;Var len Qry Name txt
DNSD_MinSZ			EQU	UDP_PKT_HDRSIZE+17 ;Min octets in DNS qry
;=====================================================================
;
;  Bit masks for TCP flags at label TCP_PKT_FLAGS
;
;=====================================================================
TCP_DAT			equ  M_BIT0
TCP_ECE			EQU  M_BIT1
TCP_URG			equ  M_BIT2
TCP_ACK			equ  M_BIT3
TCP_PSH			equ  M_BIT4
TCP_RST			equ  M_BIT5
TCP_SYN			equ  M_BIT6
TCP_FIN			equ  M_BIT7
TCP_RST_ACK			EQU  TCP_RST+TCP_ACK
TCP_SYN_FIN			EQU  TCP_SYN+TCP_FIN
TCP_FIN_ACK			EQU  TCP_FIN+TCP_ACK
;=====================================================================
;  TCP/IP Protocols
;=====================================================================
IP_PRTCL_ICMP    		EQU  1
IP_PRTCL_IGMP    		EQU  2
IP_PRTCL_TCP     		EQU  6
IP_PRTCL_UDP     		EQU  17
IP_PRTCL_OSPF    		EQU  89
IP_PRTCL_SCTP    		EQU  132
IP_PRTCL_HDOS		EQU  21	;Piggy back onto FTP
;=====================================================================
;  Fragmentation values
;=====================================================================
IP_DF             	equ  &40	; Values for IP_FRG
IP_MF             	equ  &20
;=====================================================================
;  Default Time-To-Live (TTL)
;  See RFC1122 (3.2.1.7) and RFC1700
;=====================================================================
IP_DEFAULT_TTL    	equ 64
;=====================================================================
;  Loop back address i.e 127.0.0.1
;=====================================================================
LOOPBACK_ADDR     	equ 127

