;===============================================================================
;
; Data structures for the HTML rom.
; Copyright (c) 2010 Ray Palmer
;
;===============================================================================
;
;  Forms Input Database Structure
;  ------------------------------
;
;  S_HTMLVarTyp  = Type of input
;                  Bit  Description
;                  0-3  Type of input
;                         0 = Check box
;                         1 = Radio Button
;                         2 = Text Input
;                         3 = Password Input
;                         4 = Submit button
;                         5 = Reset
;                         6 = File
;                         7 = Image
;                         8 = Hidden
;                         9 = Text Box
;                        10 = Select Flag
;                        11 = Option Grouping flag
;                        12 = Option item
;                        13 = Custon button
;                        14 = Unused/Reserved.
;                        15 = Forms Start Flag.
;                   4    Display Status (0 = No Show, 1 = Show)
;                   5    Multi-Select state (0=Single, 1=Many)
;                   6    Masked Text (0=Normal, 1= Masked)
;                   7    DB entry status (0=New, 1=Rendered)
;                  255 = end of database
;  S_HTMLVarRow  = web-page canvas row when rendered
;                  0 = unassigned row
;  S_HTMLVarCol  = web-page canvas col when rendered
;                  0 = unassigned column
;  S_HTMLVarNR   = number of rows for item displayed on web-page.
;                  Max value is 255
;  S_HTMLVarNC   = number of columns for item displayed on web-page.
;                  Max value is 255
;  S_HTMLVarVal  = Flags to control the state of the Forms Item input.
;                  Bit  Description
;                   7   Read/Write (0=Read/Write, 1=Read Only)
;                   6   Active state (0=Down, 1=Up/active)
;                   5   Text Input (0= single line, 1=multi-line)
;                   4   Item uses a label as text
;                   3   Select control (0=single, 1=multiple)
;                   2   Text entered (0=None, 1=Text entered by user)
;                   1   Internal Use Only
;                   0   Item in Group flag (0=No, 1=Yes)
;  S_HTMLVARIni  = initial non-text input value (see S_HTMLVarVal for bits
;                  set/cleared for initial value)
;  S_HTMLVarLoc  = Location of input in the main memory (&4000-&7fff)
;                  or address of menu list in HTML data.
;
;                  May also be an address of a script to execute in a
;                  RAM bank (it may also be a binary executable)
;
;                  if the value is &FFFF &FFFF then no address was
;                  allocated if TEXT, PASSWORD or TEXTAREA.
;  S_HTMLVarAdr  = Address within HTML for the start of the command
;  S_HTMLVarID   = ID number for multiple forms
;  S_HTMLVarOpt  = Option grouping ID number
;                  =0 when option is not part of a group.
;  S_HTMLVarTab  = Tabbing index (default is previous+1)
;  S_HTMLVarPrc  = Processing Flags
;                  Bit  Description
;                   7   Post/Get (0=Post, 1=Get)
;                   6   encode type (0=urlencoded,1=multipart/form-data)
;                  5-0  unused/reserved
;  S_HTMLVarSID  = Selection ID number
;                  =0 when option is not part of a group.
;
; The ESC key is used to exit out of text input to return to normal
; web-page cursor control.
;
;=========================================================================
S_HTMLVar	EQU	0
S_HTMLVarTyp	EQU	S_HTMLVar	;Type of input
S_HTMLVarRow	EQU	S_HTMLVarTyp+1	;start screen row on render
S_HTMLVarCol	EQU	S_HTMLVarRow+2	;start screen col on render
S_HTMLVarNR	EQU	S_HTMLVarCol+2	;number of rows for item display
S_HTMLVarNC	EQU	S_HTMLVarNR+1	;number of cols for item display
S_HTMLVarVal	EQu	S_HTMLVarNC+1	;value of non-text input
S_HTMLVARIni	EQU	S_HTMLVarVal+1	;initial value for non-text
S_HTMLVarLoc	EQU	S_HTMLVARIni+1	;address for text input/script
S_HTMLVarAdr	EQU	S_HTMLVarLoc+4	;address in html for command
S_HTMLVarID	EQU	S_HTMLVarAdr+3	;Form number.
S_HTMLVarOpt	EQU	S_HTMLVarID+1	;Option grouping id
S_HTMLVarTab	EQU	S_HTMLVarOpt+1	;Tab index
S_HTMLVarPrc	EQU	S_HTMLVarTab+1
S_HTMLVarSID	EQU	S_HTMLVarPrc+1
;
S_HTMLVarSz	EQU	S_HTMLVarSID+1	;size of structure.
;=========================================================================
;
;  Memory Allocation manager structure
;  -----------------------------------
;
;  S_HTMLMemFlg = Flags of memory allocations
;                 Bit  Description
;                  7   Memory availability (0=Free, 1=Taken)
;                  6   End of Allocations (0=No, 1=Yes)
;                 5-0  Unused/Reserved
;  S_HTMLMemAdd = Addrss of allocated memory
;  S_HTMLMemBnk = Bank index to allocation
;                 Actual number of Bank to swap in
;
;  The browser is limited in the vector of allocations, so it is possible
;  that the available use of many input items for text may run out of
;  available buffers.
;=========================================================================
S_HTMLMem	EQU	0
S_HTMLMemFlg	EQU	S_HTMLMem	;Allocation flags
S_HTMLMemAdd	EQU	S_HTMLMemFlg+1	;Address of allocation
S_HTMLMemBnk	EQU	S_HTMLMemAdd+2	;Bank of memory given
;
S_HTMLMemSz	EQU	S_HTMLMemBnk+1	;size of structure.
;=========================================================================
;
;  Frames Database Structure
;  -------------------------
;
;  S_HTMLFramID = Frames ID Flags (0 = End of DB)
;                   Bit  Description
;                    7   lock flag (1=locked)
;                        if set then re-size denied
;                    6   scroll flag (0=No, 1=Yes)
;                    5   position (0=fixed, 1= dynamic)
;                        (Flag only for IFRAMEs)
;                    4   Frame border (0 = no border, 1 = show border)
;                    3   Frame visibility (0=Yes, 1=No)
;                        if set then web-page window cannot show frame.
;                    2   Frame files load status (0=no content, 1=request)
;                    1   Frame content load failed (0=loaded, 1=failed)
;                    0   Frame entry free status (0=free, 1= taken)
;
;  S_HTMLFramSF = Frame Status Flags (operation use only)
;                   Bit  Description
;                   7-1  Unused/Reserved
;                    0   Frame is to be refreshed
;  S_HTMLFramBr = Frames Base Row
;  S_HTMLFramBc = Frames Base Column
;  S_HTMLFramNr = Frames End Row
;  S_HTMLFramNc = Frames End Column
;  S_HTMLFramRo = Row offset to content if scrollable.
;                 Max offset is 32767 (since max width is 65535)
;  S_HTMLFramCo = Column offset to content if scrollable.
;                 Max offset is 32767 (since max width is 65535)
;  S_HTMLFramAd = address of HTML data or HTML Command
;
;  Frames ID = 0 main menu screen
;              1 main default web-page
;
;  The first two entries are valid CPC screen definitions for use by
;  the actual screen redirections.
;
;  The frame ID is also used to rename the downloaded HTML to FrameXX.htm
;=========================================================================
S_HTMLFram	EQU	0
S_HTMLFramID	EQU	S_HTMLFram	;Frames Status Flags #1
S_HTMLFramSF	EQU	S_HTMLFramID+1	;Frames Status Flags #2
S_HTMLFramBr	EQU	S_HTMLFramSF+1	;Frames base Row
S_HTMLFramBc	EQU	S_HTMLFramBr+2	;Frames Base Column
S_HTMLFramNr	EQU	S_HTMLFramBc+2	;Frames Num Rows/End Row
S_HTMLFramNc	EQU	S_HTMLFramNr+2	;Frames Num Columns/End Column
S_HTMLFramRo	EQU	S_HTMLFramNc+2	;Frames scrolled row offset
S_HTMLFramCo	EQU	S_HTMLFramRo+2	;Frames scrolled column offset
S_HTMLFramAd	EQU	S_HTMLFramCo+2	;HTML location
;
S_HTMLFramSz	EQU	S_HTMLFramAd+3	;Size of structure
;