;===============================================================================
;
; Application Variables for the HTML rendering commands/routines
; Copyright (c) 2010 Ray Palmer
;
; Notes
; 1. Ensure the label HTML_StartVars is defined to denote where the HTML
;    variables are.
;
;===============================================================================
;
;  VARIABLES FOR GENERAL APPLICATION PROCESSING.
;
;  HTML_CmdData = HTML command type structure
;
;  The format of the commands in the table the browser processes is as follows -
;   1. Command text such as "<xxxxxxx>" without the '<' and '>'
;   2. end command byte (with bit 7 set and flag set/cleared as required)
;   3. address to handle the command
;   4. Memory type byte for the handler
;
;   Items 2, 3 and 4 are as follows -
;
;   Byte  Description
;    0    Command Table information type
;         Bit  Detials
;         7   Fixed at 1.
;         6   Command has a matching end HTML command (0=No, 1=Yes)
;             e.g </html> ...... </html>
;         5   Command can be handled by browser (0=No, 1=Yes)
;         4   Command may have additional data following (0=No, 1=Yes)
;             e.g object data for jpgs, java script, etc
;         3   Command restrictions (0=anytime, 1=head only)
;         2   End command flag (0=required, 1=Optional)
;             if Bit 6 = 0 then bit 2 is ignored.
;         1   Command has style information (0=none, 1=allowed)
;         0   unused/reserved
;
;   1-2   Handler address (0=No handler)
;    3    Memory type for handler
;         Bits 5-0 define attributes for type
;         Bits 7/6 define memory type (main/rom/banked/4Mb)
;                  = &00 then handler is in first 64K
;                  = &01 then handler is in a ROM (ROM number bits 5-0)
;                        There can only be a max of 32 roms addressable.
;                  = &10 then handler is in banked memory above dktronics.
;                        bits 5-0 has index into list of banks. This
;                        has potential for extra 512 Kbytes of memory.
;                  = &11 then handler is in dk'tronics bank
;                        D'ktronics bank number is in bits 5-0
;
;  Example -
;
;     defb  "testcommand"
;     defb  &80
;     defw  command_handler
;     defb  0
;
;  When the current position in the html file encounters the < character, the
;  remaining characters required for command checks are extracted to a local
;  variable (if required chars are upto the first space).
;
;  Any command which cannot be handled will be skipped and if there is any data
;  associated with the command then this may be seen in the rendered page.
;
;  HTML_ObjLocn = 3 byte location data for the HTM_GetCmd routine.
;
;  HTML_CurrLoc = current location in the file of the HTML data being loaded
;                 into the system.
;  HTML_SkipFlag = loading skip flag to not load commands that cannot be
;                  handle by the browser.
;  HTML_CurrBnkIdx = contains the current bank index when loading the html
;                    file data into the system.
;
;  HTML_CmdStart = Start location of the HTML command in the HTML file.
;  HTML_CmdEnd   = end location of the HTML command in the HTML file.
;
;  HTML Bank Location data is a 3 byte structure as follows -
;	Byte(s)	Description
;	 0-1		Bank memory address
;	  2		HTML Bank Index
;
;  HTML_ObjStart = start location of the HTML object in the HTML file.
;  HTML_ObjEnd   = end location of the HTML object in the HTML file.
;  The start and end locations are for commands such <style> ... </style>.
;  The primary use of the object start and end is for tables.
;
;  HTML_FCurrPos = Current address into the HTML data for the Find function.
;                  .Address = Location of found text
;                  .BankIndex = RAM Bank Number
;                               Bit 7 = Find/FindNext Flag
;                               if 255 then no find required.
;  HTML_FCurrPosF = frame of foudn text or #label
;  HTML_FCurrPosT = type adjustment to HTML_FCurrPos
;                   Bit  Description
;                    7   Flag to adjust for #label (1=Adjust, 0 = none)
;                    6   Flag to adjust for normal text found
;                   5-0  Unused
;
;  HTML_TextType = type of text to search for.
;                  0 = search within HTML commands
;                  1 = search displayable text
;
;  HTML_RenderState = Current State of HTML page rendering
;                     Bit  Description
;                      7   end of rendering when row in web-page exceeds the
;                          screen offsets.
;                      6   Char display flag (0=cannot display char else 1)
;                      5   Rendering Control Flag (0=None, 1=Active)
;                      4   Page window refresh (0=No, 1=Yes)
;                      3   Information gathering flag (0=Show, 1=Acquire)
;                      2   Table deterministic flag (0=No, 1=Yes)
;                      1   Font type to use (0=system ,1=loaded font)
;                      0   Rendering start flag (0=No, 1=Yes)
;
;  HTML_DispStatus = Current display status during rendering.
;                     Bit  Description
;                      7   position is before top row of screen
;                      6   position is after bottom row of screen.
;                      5   position is before left column of screen
;                      4   position is after right column of screen.
;                     3-1  Unused/reserved
;                      0   Complete Screen refresh (includes menus)
;
;  HTML_RCurrPos = Current start position into the HTML data which corresponds
;                  to current top row shown on screen during rendering.
;                  This variable is adjusted for page up and down scrolling.
;                  Page up scrolling is adjusted by 10 lines while page down
;                  is by 1 line at a time.
;                  When page up scorlling is down the cursor is reset to top
;                  left of window when scrolling is done.
;
;  HTML_Col/HTML_Row = top left corner of view onto the web page during
;                      page rendering. These vars are working vars during
;                      the entire web page rendering. When rendering, if the
;                      Row var exceeds the ERowOff var then rendering stops.
;
;  HTML_FontType = Font type
;                  0 = fixed space fonts
;                  1 = proportinal font.
;
;  HTML_FontEffects = Bit flag of effects to apply to the current font.
;                     Bit  Description
;                      7 = Unused
;                      6 = Unused
;                      5 = Unused
;                      4 = Over-strike
;                      3 = Mask text with * for hidden output.
;                      2 = Inverse Text
;                      1 = Strike-Through
;                      0 = Underline
;
;  HTML_CmdID = The number of the command found in the valid list DB.
;  HTML_CmdLoc = temp variable to quickly access the HTML_CmdStates data
;  HTML_CmdStates = state of each HTML command downloaded before each
;                   render of a frame or base HTML content.
;
;  HTML_PreserveFlag = state of rendering the HTML data
;                      Bit Description
;                       7  State of what the rendering is to take place.
;                          0 = HTML file, 1 = Non-HTML (ignore any HTML cmds)
;                      6-2 Unused/reserved
;                       1  Font to use (0=System, 1=use loaded font).
;                       0  HTML command <pre> encountered.
;
;===============================================================================
HTML_CmdData		EQU	HTML_StartVars
HTML_ParamNum		EQU	HTML_CmdData+4
HTML_ParamTyp		EQU	HTML_ParamNum+1
HTML_ParamLoc		EQU	HTML_ParamTyp+1
HTML_ParamVLoc		EQU	HTML_ParamLoc+2
HTML_ObjLocn		EQU	HTML_ParamVLoc+2
HTML_CurrLoc		EQU	HTML_ObjLocn+3
HTML_SkipFlag		EQU	HTML_CurrLoc+3
HTML_CurrBnkIdx		EQU	HTML_SkipFlag+1
HTML_CmdStart		EQU	HTML_CurrBnkIdx+1
HTML_CmdEnd		EQU	HTML_CmdStart+3
HTML_ObjStart		EQU	HTML_CmdEnd+3
HTML_ObjEnd		EQU	HTML_ObjStart+3
HTML_ObjERow		EQU	HTML_ObjEnd+3
HTML_ObjECol		EQU	HTML_ObjERow+2
HTML_FCurrPos		EQU	HTML_ObjECol+2
HTML_FCurrPosF		EQU	HTML_FCurrPos+3
HTML_FCurrPosT		EQU	HTML_FCurrPosF+1
HTML_TextType		EQU	HTML_FCurrPosT+1
HTML_RenderState	EQU	HTML_TextType+1
HTML_DispStatus		EQU	HTML_RenderState+1
HTML_CmdAddr		EQU	HTML_DispStatus+1
HTML_RCurrPos		EQU	HTML_CmdAddr+2
HTML_Col		EQU	HTML_RCurrPos+3
HTML_Row		EQU	HTML_Col+2
HTML_SCol		EQU	HTML_Row+2
HTML_SRow		EQU	HTML_SCol+2
HTML_MaxCol		EQU	HTML_SRow+2
HTML_FontType		EQU	HTML_MaxCol+2
HTML_FontEffects	EQU	HTML_FontType+1
HTML_TxtAlign		EQU	HTML_FontEffects+1
HTML_CmdID		EQU	HTML_TxtAlign+1
HTML_CmdLoc		EQU	HTML_CmdID+1
HTML_CmdStates		EQU	HTML_CmdLoc+2
HTML_PreserveFlag	EQU	HTML_CmdStates+NumHTMLCommands
HTML_E_VrsGen		EQU	HTML_PreserveFlag+1
;===============================================================================
;
;  VARIABLES FOR TEXT STRINGS.
;
;  HTML_CmdTemp = Contains the first n bytes (max 32) of command data that is
;                 required to enable the search to determine if the command
;                 is processable.
;
;  HTML_LblTemp = The variable will also contain the label (max 32 chars)
;                 to find when a URL has one (i.e #label)
;
;  HTML_ObjTemp = Contains the first n bytes (max 32) of command data used to
;                 Locate the / end command. The variable will also contain
;                 the label to find when a URL has one (i.e #label)
;
;===============================================================================
HTML_CmdTemp		EQU	HTML_E_VrsGen
;
HTML_LblTemp		EQU	HTML_CmdTemp+2
HTML_LblTempL		EQU	HTML_LblTemp+HTML_ObjTempSz+1
;
HTML_ObjTemp		EQU	HTML_LblTempL+1
HTML_ObjTempL		EQU	HTML_ObjTemp+HTML_ObjTempSz+1
;
HTML_TmpStr		EQU	HTML_ObjTempL+1
;
HTML_E_VrsTxt		EQU	HTML_TmpStr+8
;===============================================================================
;
;  VARIABLES CONTROLLING CONTENT RENDERING OF HTML
;
;  HTML_html = flag indicating start and end of <HTML> commands
;              (Upper), Lower is used for Comment handling.
;              Comments delimited by '--'.
;  HTML_headbody = flag indicating start and end of <HEAD> commands
;                  (lower)
;                  flag indicating start and end of <BODY> commands
;                  (upper)
;                 for either flag the lowest bit indicates end command
;                 while the upper most bit indicates start command.
;  HTML_quoteLevel = level of rendering for quotes (0=double >0 = single)
;  HTM_WrapLocks = locks controlling page scrolling and text wrapping.
;                  Bit  Description
;                   7   Wrap text lock Setting (0=No wrap, 1=wrapped)
;                   6   Text type flag (0=Normal, 1=Table data)
;                  5-0  Unused/Reserved
;  HTML_dtype = doc type of HTML rendering (0=strict,1=loose,2=frameset)
;
;===============================================================================
HTML_html		EQU	HTML_E_VrsTxt
HTML_headbody		EQU	HTML_html+1
HTML_quoteLevel		EQU	HTML_headbody+1
HTML_dtype		EQU	HTML_quoteLevel+1
HTML_WrapLocks		EQU	HTML_dtype+1
HTML_E_VrsHTM		EQU	HTML_WrapLocks+1
;===============================================================================
;
;  VARIABLES HANDLING OF LISTS
;
;  HTML_ListIdnt = Indentation for levels.
;
;  HTML_ListNum = List number for order items
;
;  HTML_ListTyp = list type flags
;                 Bit  Description
;                 7-2  Unused
;                  1   ordered list
;                  0   Unordered list
;
;===============================================================================
HTML_ListIdnt		EQU	HTML_E_VrsHTM
HTML_ListNum		EQU	HTML_ListIdnt+1
HTML_ListTyp		EQU	HTML_ListNum+1
HTML_E_VrsLst		EQU	HTML_ListTyp+1
;===============================================================================
;
;  VARIABLES FOR HANDLING OF TABLES
;
;  HTML_TblState = bit flags/status of table processing.
;                  Bit  Description
;                   7   Unused
;                   6   TR occurence (0=first, 1=more than 1)
;                   5   Table info processing (0=Underway, 1=Info got)
;                   4   Table row flag
;                       0 = first row of columns being determined
;                       1 = first of columns known.
;                   3   Table columns status flag
;                       0 = getting widths, 1 = widths now known
;                   2   Table cells have a border.
;                   1   Start Table display flag (1=table in screen, else 0)
;                       if any row of table is within screen then this flag is 1.
;                   0   Table display flag (1=displaying table data now, else 0)
;                       This flag is used to prevent premature ending of rendering
;                       when current column test is below bottom of screen.
;
;  HTML_TblAlign = Whole Table alignment
;  HTML_Tblwidth = Whole Table width
;  HTML_TblMaxRows = Number of <TR> encountered
;  HTML_TblMaxCols = Max Num <TD>'s between <TR>'s.
;  HTML_TblCurrRow = Current Tbl Formats row
;  HTML_TblCurrCol = Current Tbl Formats column
;  HTML_TblIndex = Index into formats/row info
;  HTML_TblSRY = Start Tbl row screen row
;  HTML_TblSRX = Start Tbl row screen column
;  HTML_TblCRY = Calculated Tbl row screen row
;  HTML_TblCCX = Calculated Tbl col screen column
;  HTML_TblNRY = Next scrn row (for data wraps)
;  HTML_TblHdrLoc = Location of <THEAD>
;  HTML_TblFtrLoc = Location of <TFOOT>
;  HTML_TblBdyCnt = number of <TBODY>
;  HTML_colgrpalign = alignent of columns in group
;  HTML_colgrpspan = number of columns in group
;  HTML_colgrpwid = width of columns in group
;  HTML_colgrpcc = current col in groups.
;  HTML_colalign = column alignent
;  HTML_colrspan = TH/TD row span
;  HTML_colcspan = TH/TD col span
;  HTML_colcwrap = TH/TD cell wrap control
;  HTML_colwid = column width
;  HTML_CurCelWid = Current cell width
;  HTML_CapAlign = Caption alignment.
;  HTML_CapSize = Caption length (max 255 chars)
;  HTML_CapLocn = Location of caption text
;  HTML_TblSRow = Table start row (work var)
;  HTML_TblNumRR = Num rows of complete row.
;  HTML_TblCapEdg = where caption is shown on the table
;                   0 = top of table
;                   1 = bottom of table
;                   2 = left edge of table (vertical caption, Top-Down)
;                   3 = right edge of table (vertical caption, Top-Down)
;  HTML_TCurrPos = RAM location after the end of the <TABLE....> command
;
;===============================================================================
HTML_TblState		EQU	HTML_E_VrsLst
HTML_TblAlign		EQU	HTML_TblState+1
HTML_Tblwidth		EQU	HTML_TblAlign+1
HTML_TblMaxRows		EQU	HTML_Tblwidth+2
HTML_TblMaxCols		EQU	HTML_TblMaxRows+2
HTML_TblCurrRow		EQU	HTML_TblMaxCols+2
HTML_TblCurrCol		EQU	HTML_TblCurrRow+2
HTML_TblIndex		EQU	HTML_TblCurrCol+2
HTML_TblSRY		EQU	HTML_TblIndex+2
HTML_TblSRX		EQU	HTML_TblSRY+2
HTML_TblCRY		EQU	HTML_TblSRX+2
HTML_TblCCX		EQU	HTML_TblCRY+2
HTML_TblNRY		EQU	HTML_TblCCX+2
HTML_TblHdrLoc		EQU	HTML_TblNRY+2
HTML_TblFtrLoc		EQU	HTML_TblHdrLoc+3
HTML_TblBdyCnt		EQU	HTML_TblFtrLoc+3
HTML_colgrpalign	EQU	HTML_TblBdyCnt+1
HTML_colgrpspan		EQU	HTML_colgrpalign+1
HTML_colgrpwid		EQU	HTML_colgrpspan+1
HTML_colgrpcc		EQU	HTML_colgrpwid+1
HTML_colalign		EQU	HTML_colgrpcc+1
HTML_colrspan		EQU	HTML_colalign+1
HTML_colcspan		EQU	HTML_colrspan+1
HTML_colcwrap		EQU	HTML_colcspan+1
HTML_colwid		EQU	HTML_colcwrap+1
HTML_CurCelWid		EQU	HTML_colwid+1
HTML_CapAlign		EQU	HTML_CurCelWid+1
HTML_CapSize		EQU	HTML_CapAlign+1
HTML_CapLocn		EQU	HTML_CapSize+1
HTML_TblSRow		EQU	HTML_CapLocn+3
HTML_TblNumRR		EQU	HTML_TblSRow+2
HTML_TblCapEdg		EQU	HTML_TblNumRR+2
HTML_TCurrPos		EQU	HTML_TblCapEdg+1
HTML_TblTemp		EQU	HTML_TCurrPos+3
HTML_TblCellWidth 	EQU	HTML_TblTemp+2
HTML_E_VrsTbls		EQU	HTML_TblCellWidth+1
;===============================================================================
;
;  VARIABLES FOR LINKS AND HYPERLINK HANDLING
;
;  HTML_href = State/Flags of <A HREF...> during rendering.
;              Bit  Description
;               7   Link object load flag (0=no, 1=Load now)
;              6-1  Unused/Reserved
;               0   Status of Hyperlink processing (0=end, 1=start)
;              Contains DB index during hyperlink search.
;  HTML_CurCol = Current Cursor Column select was acted on.
;  HTML_CurRow = Current Cursor Row select was acted on.
;  HTML_HLDbItem = temp store for hyperlink DB data.
;
;===============================================================================
HTML_href		EQU	HTML_E_VrsTbls
HTML_CurCol		EQU	HTML_href+1
HTML_CurRow		EQU	HTML_CurCol+2
HTML_WPHLIndex		EQU	HTML_CurRow+2
HTML_DBHLAddr		EQU	HTML_WPHLIndex+2
HTML_HLDbItem		EQU	HTML_DBHLAddr+3
HTM_E_VrsHypL		EQU	HTML_HLDbItem+S_HLSz
;===============================================================================
;
;  VARIABLE FOR HANDLING CONTENT POSITIONING.
;
;  HTML_Indent = Indent position for Lists
;                Incremented by 5 chars for new list type and
;                Decremented for each end of list type.
;
;  HTML_BR = start column for non-frames and frame HTML output.
;  HTML_BR_Table = Start offset from the HTML_BR for data display within table
;                  cells for both header and normal data.
;                  Var is setup upon a <TD> or <TH> command
;  HTML_BR_FormsT = Start offset from the HTML_BR for forms multi-line input.
;                   For single line input there is no offset.
;  HTML_BR_List = start column offset for lists.
;  HTML_BR_Flag = Flag controlling new line for objects such as images
;                 Bit  Description
;                 7-6  BR command handle type
;                       00 - None (default)
;                       01 - Right (new line below object on right margin)
;                       10 - Left (new line below object on left margin)
;                       11 - All (new line below any object)
;                 5-3  Unused/Reserved
;                  2   BR Command has new start row if object wrap of text
;                  1   BR command directs text to start from right of object
;                      when row is below object, bit 1 and 2 are cleared.
;                  0   Unused/Reserved
;
;===============================================================================
HTML_Indent		EQU	HTM_E_VrsHypL
HTML_BR			EQU	HTML_Indent+2
HTML_BR_Table		EQU	HTML_BR+2
HTML_BR_FormsT		EQU	HTML_BR_Table+2
HTML_BR_List		EQU	HTML_BR_FormsT+2
HTML_BR_DL		EQU	HTML_BR_List+2
HTML_BR_Row		EQU	HTML_BR_DL+2
HTML_BR_Col		EQU	HTML_BR_Row+2
HTML_BR_Flag		EQU	HTML_BR_Col+2
HTML_ImgRow		EQU	HTML_BR_Flag+1
HTML_ImgCol		EQU	HTML_ImgRow+2
HTML_E_VrsCurP		EQU	HTML_ImgCol+2
;===============================================================================
;
;  VARIABLES FOR FORMS HANDLING
;
;  HTML_FormsPF = Forms input processing flags.
;                    Bit  Description
;                     7   Form rendering status (0=No, 1=Yes)
;                     6   Unused
;                     5   Unused
;                     4   Unused
;                     3   Unused
;                     2   Unused
;                     1   Processing flag (0=Started, 1=Underway)
;                     0   Form has border (0=No, 1=Yes)
;  HTML_FormsFlag = bit flags of handling Forms - input
;                    Bit  Description
;                     7   Gathering forms information (1=Yes, 0=No)
;                     6   item active state
;                     5   Item has label for text
;                     4   selected flag (0=no, 1=Yes)
;                     3   mulitple selection enabled (0=single, 1=many)
;                     2   Group items flag (0=No, 1=Yes in group)
;                     1   Option Group active (0=No, 1=Yes)
;                     0   Forms (refresh State) (0=None, 1=refresh)
;
;  HTML_FormsObj = Local copy of the Forms Database entry 
;                  (structure definition in cpcb-s.inc)
;
;===============================================================================
HTML_FormsPF		EQU	HTML_E_VrsCurP
HTML_FormsFlag		EQU	HTML_FormsPF+2
HTML_TextStrMax		EQU	HTML_FormsFlag+1
HTML_TextStrlen		EQU	HTML_TextStrMax+1
HTML_TextLinNum		EQU	HTML_TextStrlen+1
HTML_TextAddr		EQU	HTML_TextLinNum+1
HTML_TextMode		EQU	HTML_TextAddr+3
HTML_FrmsDBIx		EQU	HTML_TextMode+1
HTML_FrmsDBIx2A		EQU	HTML_FrmsDBIx+2
HTML_FrmsDBLocn		EQU	HTML_FrmsDBIx2A+2
HTML_FrmsIDNum		EQU	HTML_FrmsDBLocn+2
HTML_FrmsMalloc		EQU	HTML_FrmsIDNum+1
;
HTML_FormsObj		EQU	HTML_FrmsMalloc+2
HTML_FrmsTyp		EQU	HTML_FormsObj+S_HTMLVarTyp
HTML_FrmsBRow		EQU	HTML_FormsObj+S_HTMLVarRow
HTML_FrmsBCol		EQU	HTML_FormsObj+S_HTMLVarCol
HTML_FrmsNRows		EQU	HTML_FormsObj+S_HTMLVarNR
HTML_FrmsNCols		EQU	HTML_FormsObj+S_HTMLVarNC
HTML_FrmsValue		EQU	HTML_FormsObj+S_HTMLVarVal
HTML_FrmsIniVal		EQU	HTML_FormsObj+S_HTMLVARIni
HTML_FrmsTxtLoc		EQU	HTML_FormsObj+S_HTMLVarLoc
HTML_FormsAddr		EQU	HTML_FormsObj+S_HTMLVarAdr
HTML_FrmsID		EQU	HTML_FormsObj+S_HTMLVarID
HTML_FrmsOptG		EQU	HTML_FormsObj+S_HTMLVarOpt
HTML_FrmsTab		EQU	HTML_FormsObj+S_HTMLVarTab
HTML_FrmsPrc		EQU	HTML_FormsObj+S_HTMLVarPrc
HTML_FrmsSID		EQU	HTML_FormsObj+S_HTMLVarSID
;
HTML_FrmsTmp		EQU	HTML_FormsObj+S_HTMLVarSz
HTML_FrmsSRow		EQU	HTML_FrmsTmp
HTML_FrmsSCol		EQU	HTML_FrmsTmp+2
HTML_FrmsERow		EQU	HTML_FrmsTmp+4
HTML_FrmsECol		EQU	HTML_FrmsTmp+6
HTML_FrmsTemp		EQU	HTML_FrmsTmp+8
HTML_FormsLocn		EQU	HTML_FrmsTemp+3
HTML_E_VrsFrms		EQU	HTML_FormsLocn+3
;===============================================================================
;
;  VARIABLES FOR FRAMES HANDLING
;
;  HTML_FrmeSets = frameset control (Upper=Rows,Lower=Columns)
;                  Values Detials
;                  &55    The rows/columns means the webpage is broken into
;                         rows x columns
;                  &15    The columns are defined first and following row
;                         definitions define the successive column breakdowns
;                  &51    The rows are defined first and following columns
;                         definitions define the successive row breakdowns
;                  &22    No frames allowed (setup via configuration/preference)
;                  Bit 7  Frames defs in use.
;                  Bit 2/6 set upon first frameset command to allow for
;                         multilpe framset command usage.
;                  Bit 1/5 not used
;
;  HTML_FrmeSetI  = index to array for rows
;  HTML_FrmeSetIC = index to array for columns
;
;  Row/column values = if values are % then upper is &80, lower = value
;                      if values are * then upper is &40, lower = value
;                      if values are # then value is &0xxx, where bits 13-0
;                        is the number of characters (ie. 16383 characters)
;                      if neither then Upper/lower = pixels -> chars
;                        bits 15/14 are set so max pixels is 16383
;
;
;  HTML_BordrSt = Flags for border processing.
;                  Bit  Description
;                   7   Border proc flag (0=None, 1=Underway)
;                   6   Border processing underway
;                   5   Start/end flag (0=Start, 1=end)
;                   4   Unused/Reserved
;                   3   Unused/Reserved
;                   2   <P> has border (0=No, 1=Yes)
;                   1   Unused/Reserved
;                   0   Form has border (0=No, 1=Yes)
;
;===============================================================================
HTML_FrmeSets		EQU	HTML_E_VrsFrms
HTML_FrmeSetN		EQU	HTML_FrmeSets+1
HTML_FrmeSetIC		EQU	HTML_FrmeSetN+1
HTML_FrmeSetI		EQU	HTML_FrmeSetIC+1
HTML_FrmeCCtr		EQU	HTML_FrmeSetI+1
HTML_FrmeRCtr		EQU	HTML_FrmeCCtr+1
HTML_FrmeSetR		EQU	HTML_FrmeRCtr+1
HTML_FrmeSetC		EQU	HTML_FrmeSetR+2
HTML_FrameLoc		EQU	HTML_FrmeSetC+2
HTML_FrameID		EQU	HTML_FrameLoc+3
HTML_FrameCurrID	EQU	HTML_FrameID+1
HTML_FrmeICtr		EQU	HTML_FrameCurrID+1	;Temp
HTML_FrameTmp		EQU	HTML_FrmeICtr+2		;Temp
HTML_FrameTmp2		EQU	HTML_FrameTmp+2		;Temp
HTML_FrameTmp3		EQU	HTML_FrameTmp2+2	;Temp
HTML_FrameTmp4		EQU	HTML_FrameTmp3+2	;Temp
HTML_FrameBS		EQU	HTML_FrameTmp4+2	;Temp
HTML_BordrSt		EQU	HTML_FrameBS+1		;border processing flgs
HTML_E_VrsFram		EQU	HTML_BordrSt+1
;===============================================================================
;
;  VARIABLES FOR HANDLING STYLE COMMANDS
;
;  HTML_StyleDb = Pointer in the styles Database of <Style> html cmds
;  HTML_NumStyles = number of items in the styles Db.
;  HTML_StyleIdx = current styles db index
;  HTML_StyleDbIdx = current HTML address of style command to process.
;  HTML_CurrHxDbIdx = index into the Hx databas and general flags
;                     Bit  Description
;                      7   Style change status (0=no change, 1=changed)
;                      6   H6 flag (0=No, 1=style setting in place)
;                      5   H5 flag (0=No, 1=style setting in place)
;                      4   H4 flag (0=No, 1=style setting in place)
;                      3   H3 flag (0=No, 1=style setting in place)
;                      2   H2 flag (0=No, 1=style setting in place)
;                      1   H1 flag (0=No, 1=style setting in place)
;                      0   Processing flag (1=Collecting, 0=Setting)
;
;                     255 = no database entry is current.
;
;  HTML_ClassIDType = HTML parameter type passed in the command to
;                     which type of style is being requested.
;
;  HTML_StyleSetting = Database object of styles settings
;                      (see cpcb-s.inc for structure details)
;
;  HTML_StyleAddr = address within HTML memory of style command
;
;  HTML_stylDBIndx = stack index into the style settings DB
;
;===============================================================================
HTML_StyleDb		EQU	HTML_E_VrsFram
HTML_NumStyles		EQU	HTML_StyleDb+2
HTML_StyleIdx		EQU	HTML_NumStyles+1
HTML_StyleDbIdx		EQU	HTML_StyleIdx+1
HTML_StyleDbIdxB	EQU	HTML_StyleDbIdx+3
HTML_CurrHxDbIdx	EQU	HTML_StyleDbIdxB+3
HTML_ClassIDType	EQU	HTML_CurrHxDbIdx+1
HTML_StyleSetting	EQU	HTML_ClassIDType+1
HTML_StyleFlgs		EQU	HTML_StyleSetting+S_StylesFlags
HTML_StyleStatus	EQU	HTML_StyleSetting+S_StylesStatus
HTML_StyleBord		EQU	HTML_StyleSetting+S_StylesBordT
HTML_StyleBordW		EQU	HTML_StyleSetting+S_StylesBordW
HTML_StylesText		EQU	HTML_StyleSetting+S_StylesText
HTML_StylesMargL	EQU	HTML_StyleSetting+S_StylesMargL
HTML_StylesMargR	EQU	HTML_StyleSetting+S_StylesMargR
HTML_StyleFntNme	EQU	HTML_StyleSetting+S_StylesFntNme
HTML_StyleAddr		EQU	HTML_StyleSetting+S_StyleSetSz
HTML_stylSetDBIndx	EQU	HTML_StyleAddr+3
HTML_StyleCmd		EQU	HTML_stylSetDBIndx+2
HTML_StyleVrEnd		EQU	HTML_StyleCmd+1
;===============================================================================
HTML_SCurrPos		EQU	HTML_StyleVrEnd+1	;Loc'n of Script cmd
HTML_ScrptTyp		EQU	HTML_SCurrPos+3		;Script Type
;===============================================================================
HTML_P_Tmp		EQU	HTML_ScrptTyp+1		;<P> vars
HTML_P_SRow		EQU	HTML_P_Tmp+S_StartRow
HTML_P_SCol		EQU	HTML_P_Tmp+S_StartCol
HTML_P_ERow		EQU	HTML_P_Tmp+S_EndRow
HTML_P_ECol		EQU	HTML_P_Tmp+S_EndCol
;===============================================================================
HTML_TmpAddr		EQU	HTML_P_Tmp+S_CPCWdwSz
;===============================================================================
HTML_EndVars		EQU	HTML_TmpAddr+3
