;=========================================================================
;
; Application Routines (memory handler)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     MEM_SaveBank
;Purpose  To push the current memory bank onto the stack.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  BankStack = stack of current banks
;         BankIndx = index into the BankStack
;         CurrMemBank = current bank of memory swapped in.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars BankStack = updated stack of current banks
;         BankIndx = updated index into the BankStack
;Uses     None.
;Call By  TBD
;Calls    None.
;Notes    TBD
;=========================================================================
MEM_SaveBank
   PUSH  AF
   PUSH  BC
   LD    A,(CurrMemBank)
   LD    BC,(BankIndx)
   LD    (BC),A
   INC   BC
   LD    (BankIndx),BC
   POP   BC
   POP   AF
   RET
;=========================================================================
;Name     MEM_PushBank
;Purpose  To push the current memory bank onto the stack.
;In Regs  A = New bank to swap in
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  BankStack = stack of current banks
;         BankIndx = index into the BankStack
;         CurrMemBank = current bank of memory swapped in.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars BankStack = updated stack of current banks
;         BankIndx = updated index into the BankStack
;Uses     None.
;Call By  SYS_SysFnctAccess
;Calls    MEM_SwapInABank
;Notes    1. The routine does not have on the stack the next swapped in
;            stack of memory.
;=========================================================================
MEM_PushBank
   CALL  MEM_SaveBank
   JP    MEM_SwapInABank
;=========================================================================
;Name     MEM_PopBank
;Purpose  To pop the current memory bank onto the stack.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  BankStack = stack of current banks
;         BankIndx = index into the BankStack
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars BankStack = updated stack of current banks
;         BankIndx = updated index into the BankStack
;Uses     TBD
;Call By  SYS_SysFnctAccess
;Calls    MEM_SwapInABank
;Notes    1. Multiple calls to this routine may cause unknown stability
;            if the indexing exceed the stack allocation.
;=========================================================================
MEM_PopBank
   PUSH  AF
   PUSH  BC
   LD    BC,(BankIndx)
   DEC   BC
   LD    A,(BC)
   LD    (BankIndx),BC
   POP   BC
   CALL  MEM_SwapInABank
   POP   AF
   RET
;=========================================================================
;Name     Mem_GetByteData
;Purpose  To a byte of data from a banked memory block.
;In Regs  A = Memory configuration data
;         BC = ??
;         DE = ??
;         HL = Memory address to get byte data.
;         IX = ??
;         IY = ??
;In Vars  CurrMemBank = currently swapped in bank
;Out Regs A = ??
;         BC = ??
;         D = ??
;         E = byte of data
;         HL = No change
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    MEM_PushBank
;         MEM_PopBank
;Notes    None.
;=========================================================================
Mem_GetByteData
   CALL  MEM_PushBank
   LD    E,(HL)
   JP    MEM_PopBank	;return previous bank in.
;=========================================================================
;Name     Mem_GetWordData
;Purpose  To get a 16-bit word of data from a banked memory block.
;In Regs  A = Memory configuration data
;         BC = ??
;         DE = ??
;         HL = Memory address to get byte data.
;         IX = ??
;         IY = ??
;In Vars  CurrMemBank = currently swapped in bank
;Out Regs A = corrupted.
;         BC = ??
;         DE = word data
;         HL = incremented by 2
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    MEM_PushBank
;         MEM_PopBank
;Notes    None.
;=========================================================================
Mem_GetWordData
   CALL  MEM_PushBank
   LD    E,(HL)
   INC   HL
   LD    D,(HL)
   INC   HL
   JP    MEM_PopBank	;return previous bank in.
;=========================================================================
;Name     Mem_Getstr
;Purpose  To get a NUL terminated string from a banked memory block.
;In Regs  A = Memory configuration data
;         B = max length to get
;         C = ??
;         DE = Destination address.
;         HL = Memory address to get byte data.
;         IX = ??
;         IY = ??
;In Vars  CurrMemBank = currently swapped in bank
;Out Regs A = corrupted.
;         BC = ??
;         DE = updated.
;         HL = updated.
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    MEM_PushBank
;         MEM_PopBank
;Notes    1. Max string is 255 bytes
;=========================================================================
Mem_Getstr
   CALL  MEM_PushBank
Mem_Getstr1
   LD    A,(HL)
   LD    (DE),A
   OR    A
   JR    Z,Mem_Getstr2
   INC   DE
   INC   HL
   DJNZ  Mem_Getstr1
Mem_Getstr2
   JP    MEM_PopBank	;return previous bank in.
;=========================================================================
;Name     Mem_CpyMem
;Purpose  To copy upto 255 bytes of memory
;In Regs  A = Memory configuration data (not used yet)
;         B = max length to get
;         C = ??
;         DE = Destination address for data
;         HL = Source address of data
;         IX = ??
;         IY = ??
;In Vars  CurrMemBank = currently swapped in bank
;Out Regs A = corrupted.
;         BC = ??
;         DE = updated.
;         HL = updated.
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    MEM_PushBank
;         MEM_PopBank
;Notes    None
;=========================================================================
Mem_CpyMem
   CALL  MEM_PushBank
Mem_CpyMem1
   LD    A,(HL)
   LD    (DE),A
   INC   DE
   INC   HL
   DJNZ  Mem_CpyMem1
   JP    MEM_PopBank	;return previous bank in.
;=========================================================================
;Name     Mem_SetByteData
;Purpose  To set a byte of data from a banked memory block.
;In Regs  A = Memory configuration data
;         BC = ??
;         DE = ??
;         HL = Memory address to get byte data.
;         IX = ??
;         IY = ??
;In Vars  CurrMemBank = currently swapped in bank
;Out Regs A = corrupted.
;         BC = ??
;         D = ??
;         E = byte of data
;         HL = No change
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    MEM_PushBank
;         MEM_PopBank
;Notes    None.
;=========================================================================
Mem_SetByteData
   CALL  MEM_PushBank
   LD    (HL),E
   JP    MEM_PopBank	;return previous bank in.
;=========================================================================
;Name     Mem_SetWordData
;Purpose  To set a 16-bit word of data from a banked memory block.
;In Regs  A = Memory configuration data
;         BC = ??
;         DE = ??
;         HL = Memory address to get byte data.
;         IX = ??
;         IY = ??
;In Vars  CurrMemBank = currently swapped in bank
;Out Regs A = corrupted.
;         BC = ??
;         DE = word data
;         HL = incremented by 2
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    MEM_PushBank
;         MEM_PopBank
;Notes    None.
;=========================================================================
Mem_SetWordData
   CALL  MEM_PushBank
   LD    (HL),E
   INC   HL
   LD    (HL),D
   INC   HL
   JP    MEM_PopBank	;return previous bank in.
;=========================================================================
;Name     MEM_CalcCfgData
;Purpose  To calculate the configuration byte for memory setting.
;In Regs  A = ??
;         B = Block index from base block (0-127)
;         C = Base block of configuration.
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = corrupted.
;         B = corrupted.
;         C = configuration data byte
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The configuration byte output is formatted as follows -
;            Bits    Description
;            7-6     512K segment index (these bits are used in I/O addr)
;            5-3     64K segment within each 512K segment.
;            2-0     Configuration as per Dk'Tronics memory managment.
;                    0-3 = not used
;                    4-7 = swap 16K segment into &4000 memory space.
;=========================================================================
MEM_CalcCfgData
   LD    A,B
   OR    A
   RET   Z		;return base block configuration
MEM_CalcCD0
   LD    A,C
   AND   7		;mask out cfg bits (0-2)
   CP    7		;at end of 64K cfg?
   JR    NZ,MEM_CalcCD2	;Yes
   LD    A,C
   AND   &FC		;restart at cfg = 4
   ADD   A,8		;next 64K segment
   LD    C,A
   DEC   C
MEM_CalcCD2
   INC   C		;next 16K segment in 64K segment.
MEM_CalcCD1
   DJNZ  MEM_CalcCD0
   RET
;=========================================================================
;Name     MEM_ClearBank
;Purpose  To clear a bank of memory
;In Regs  A = Bank configuraton data
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = corrupted.
;         DE = corrupted.
;         HL = corrupted.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
MEM_ClearBank
   CALL  Mem_SwapInABank
   LD    HL,&4000
   LD    DE,&4001
   LD    (HL),0
   LD    BC,&3fff
   LDIR
   RET
;=========================================================================
;Name     Mem_SwapInCode
;Purpose  To swap in the code bank of memory
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = corrupted
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Only working for DK'tronics memory.
;=========================================================================
MEM_SwapInCode
   LD    A,2
;=========================================================================
;Name     MEM_SwapBCBank
;Purpose  To swap in a bank of memory for browser code.
;In Regs  AF = Code bank number index (0-127)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Only working for DK'tronics memory.
;=========================================================================
MEM_SwapBCBank
   PUSH  BC
   LD    B,A
   LD    A,(CodeBanks)		;Base block configuration
   JR    MEM_SwapInBank1
;=========================================================================
;Name     Mem_SwapInHTML
;Purpose  To swap in a HTML bank of memory
;In Regs  A = HTML bank number index (0-127)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Only working for DK'tronics memory.
;=========================================================================
MEM_SwapInHTML
   PUSH  BC
   LD    B,A
   LD    A,(HTMLBanks)		;Base block configuration
MEM_SwapInBank1
   LD    C,A
   CALL  MEM_CalcCfgData
   LD    A,C
   JR    Mem_SwapInABank2
;=========================================================================
;Name     Mem_SwapInApps
;Purpose  To swap in the Applications bank of memory
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = corrupted
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Only working for DK'tronics memory.
;=========================================================================
MEM_SwapInApps
   LD    A,(AppsBank)
   JR    Mem_SwapInABank
;=========================================================================
;Name     Mem_SwapInFramesDB
;Purpose  To swap in the bank of memory with the Frames database
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FramesDBLoc = ???
;Out Regs AF = corrupted
;         BC = ??
;         DE = ??
;         HL = address of Frames Database.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Only working for DK'tronics memory.
;=========================================================================
MEM_SwapInFramesDB
   LD    A,(FramesDBLoc+2)
   JR    Mem_SwapInABank
;=========================================================================
;Name     Mem_SwapInCfg
;Purpose  To swap in the configuration bank of memory
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = corrupted
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Only working for DK'tronics memory.
;=========================================================================
MEM_SwapInCfg
   LD    A,(ConfigBank)
   JR    Mem_SwapInABank
;=========================================================================
;Name     Mem_SwapInLink
;Purpose  To swap in the links bank of memory
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = corrupted
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Only working for DK'tronics memory.
;=========================================================================
MEM_SwapInLink
   LD    A,(LinksBank)
   JR    Mem_SwapInABank
;=========================================================================
;Name     Mem_SwapInMain
;Purpose  To swap in a the main bank of memory
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = corrupted
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
MEM_SwapInMain
   LD    A,MEMCFGDEFAULT
;=========================================================================
;Name     MEM_SwapInABank
;Purpose  To swap in a bank of memory
;In Regs  A = Bank configuration data
;             Bits 7-6 512K segment number (i.e 2Mb max)
;             Bits 5-0 Dk'Tronics configuration data.
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars CurrMemBank = new memory configuration.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
MEM_SwapInABank
   PUSH  BC
MEM_SwapInABank2
   LD    (CurrMemBank),A
;
   LD    B,A		;Save index
   AND   &3F		;Mask out Dk'Tronics configuration (bits 0-5)
   LD    C,A
   RLC   B		;Shift bits 6-7 to 0-1
   RLC   B
   LD    A,&7C
   OR    B		;Setup Upper I/O address
   LD    B,A
   LD    A,C
   LD    C,&FF		;Lower I/O address
   OR    &C0		;ensure correct mem cfg change data for I/O
   OUT   (C),A
   POP   BC
   RET