;=========================================================================
;
; Application Routines (HTML rendering commands - Table)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
; 1. The rendering of the current command does not expect that the same
;    command to be recursed. This may lead to an infinte loop.
;
; 2. Where a command changes the font (e.g normal to italic) the original
;    command will have no way of reverting the font back to what it was.
;
;=========================================================================
;
;  HTML command handlers
;
;=========================================================================
;Name     html_cmd_table
;Purpose  To handle the html command <table....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         HTML_TblState = all flags cleared on first entry.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblMaxRows = max number of rows in the table
;         HTML_TblMaxCols = max number of column in the table
;         HTML_TblHdrLoc = location of THEAD data
;         HTML_TblFtrLoc = location of TFOOT data
;         HTML_TblBdyLoc = Location of table body.
;         HTML_TblSRY = table row start
;         HTML_TblSRX = table column start
;Uses     TBD
;Call By  On detection of <TABLE...> or </TABLE> commands.
;Calls    HTM_Get1stCmdChr
;         HTM_GetTblInfo
;         HTM_SetNextRow
;         HTM_ChkTblBorder
;         HTM_ShwRowBorders
;         HTM_CaptionShow
;         HTM_GetCmdParam
;         HTM_FindParmLbl
;         HTM_SetTableWidth
;         HTM_GetCmdParmVal
;         HTM_SetTableAlign
;         HTM_FlagTableBorder
;         HTML_ResetTblData
;         HTM_SetupEndRow
;Notes    1. If the table can be wholey displayed on the screen then for
;            all left/right scrolling the table is always completely
;            displayed.
;=========================================================================
html_cmd_table
   LD    HL,0
   LD    (HTML_BR_Table),HL		;setup start col for caption.
;
   CALL  HTM_Get1stCmdChr		;</TABLE> command ?
   JP    NZ,html_cmd_table1		;No (maybe start to gather info)
;
;  Found </Table> command
;
   LD    HL,HTML_TblState
   BIT   0,(HL)				;table being rendered?
   JR    NZ,html_cmd_table0a		;Yes
   LD    A,(HL)
   AND   &04
   OR    &21
   LD    (HL),A				;All done, now rendering.
;
   LD    HL,HTML_RenderState
   RES   2,(HL)				;Flag not collection table
   RES   3,(HL)				;allow for display
;
   LD    DE,HTML_CmdEnd
   LD    HL,HTML_TCurrPos
   LD    BC,3
   LDIR					;Setup re-entry to render table.
;
   LD    HL,0
   LD    (HTML_TblIndex),HL		;formats/row/column info index
   LD    (HTML_TblCurrRow),HL		;reset table row.
   JP    HTM_GetTblInfo			;Get table render start column.
;
;  Display caption after table (if required)
;
html_cmd_table0a
   CALL  HTM_SetupEndRow
   PUSH  HL
   CALL  HTM_ChkTblBorder		;Table has borders to show?
   CALL  NZ,HTM_ShowAllBorders2
   POP   HL
   INC   HL
   LD    (HTML_Row),HL
   LD    A,(HTML_CapAlign)
   CP    HTM_ALIGNBOTTOM		;Show caption on bottom of table?
   JR    NZ,html_cmd_table0d		;No
   CALL  HTM_CaptionShow
   CALL  HTM_SetNextRow
html_cmd_table0d
   LD    HL,0
   LD    (HTML_Col),HL
   RET
;
;  Display caption before table (if required)
;
html_cmd_table0b
   CALL  HTM_SumTblWidths
   LD    HL,(HTML_TblCCX)		;Setup render start Column
   LD    (HTML_Col),HL
   LD    DE,(HTML_TblSRow)		;Get start row for render
   LD    (HTML_Row),DE
;
   LD    A,(HTML_CapAlign)
   CP    HTM_ALIGNTOP			;Show caption on top of table?
   JR    NZ,html_cmd_table0e		;No
;
   CALL  HTM_CaptionShow
   LD    HL,(HTML_Row)			;Get Current row
   INC   HL					;skip caption
;
html_cmd_table0e
   LD    (HTML_TblSRY),HL		;Get current start row
   LD    HL,0
   LD    (HTML_Col),HL
   RET
;
;  Found <Table> command
;
html_cmd_table1
   LD    HL,HTML_TblState
   BIT   0,(HL)				;table being rendered?
   JR    NZ,html_cmd_table0b		;Yes (must be re-entry)
   LD    (HL),0				;Clear all flags/status bits
;
;  Scan the parameters to <TABLE ...> command
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_table2
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    '>'				;no more parameters?
   JR    Z,html_cmd_table0		;Yes
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_Border			;Is parameter 'border' ?
   CP    B
   JR    Z,html_cmd_table3		;Yes
   LD    A,HTM_P_Align			;Is parameter 'align' ?
   CP    B
   JR    Z,html_cmd_table3		;Yes
   LD    A,HTM_P_Width
   CP    B					;Is parameter 'width' ?
   JR    NZ,html_cmd_table2		;No (skip the parameter)
   CALL  HTM_SetTableWidth		;Get table width
   JR    html_cmd_table2		;do next parameter
;
html_cmd_table3
   LD    (HTML_ParamTyp),A		;Save parameter type
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   LD    A,(HTML_ParamTyp)
   CP    HTM_P_Align			;Table web-page alignment ?
   CALL  Z,HTM_SetTableAlign		;Yes
   LD    A,(HTML_ParamTyp)
   CP    HTM_P_Border			;Table has border?
   CALL  Z,HTM_FlagTableBorder	;Yes
   JR    html_cmd_table2		;do next parameter
;
html_cmd_table0
;
;  Save current HTML location to know where to start
;  from after table information gathering has completed.
;
   LD    HL,(HTML_CmdStart)
   LD    A,(HTML_CmdStart+2)
   DEC   HL
   LD    (HTML_TCurrPos),HL
   LD    (HTML_TCurrPos+2),A
;
   LD    HL,(HTML_Row)
   LD    (HTML_TblSRow),HL		;Save start row for render
   LD    HL,(HTML_Col)
   LD    (HTML_TblCCX),HL		;Setup render start Column
;
   LD    HL,HTML_RenderState
   SET   2,(HL)				;Flag acquiring table info
   SET   3,(HL)				;Flag gathering render info
;
   LD    HL,0
   LD    (HTML_TblIndex),HL		;Start at first cell in formats table
;
   XOR   A
   LD    (HTML_TblBdyCnt),A		;Reset num <TBODY> commands
   LD    (HTML_colgrpalign),A		;default align - left
   LD    (HTML_colgrpcc),A		;grp col start offset
   DEC   A
   LD    (HTML_colgrpwid),A		;default width - 255 (max)
   LD    (HTML_colgrpspan),A		;Default span - all columns
   LD    (HTML_CapAlign),A		;Assume no caption.
;
;
;  Clear the table Information areas
;
   LD    A,&C0
   LD    HL,(HTML_TblFmtDat)
   CALL  HTML_ResetTblData		;clear all flags to default.
   XOR   A
   LD    HL,(HTML_TblColWid)
   CALL  HTML_ResetTblData		;max span of cols is 0
   LD    HL,(HTML_TblRowWid)
   CALL  HTML_ResetTblData		;max span of rows is 0
   INC   A
   LD    HL,(HTML_TblNRSWid)
   CALL  HTML_ResetTblData		;max widths of cols is 1
;
   RET
;=========================================================================
;Name     html_cmd_thead
;Purpose  To handle the html command <thead....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  On detection of <THEAD...> or </THEAD> commands.
;Calls    HTM_ChkTblState
;         TBD
;Notes    1. The table header HTML command is used primarily for the
;            hard copy output since pages are of fixed length and that
;            tables need to column titles across the pages.
;=========================================================================
html_cmd_thead
   CALL  HTM_ChkTblState		;Collecting table info?
   RET   Z					;No
   LD    DE,HTML_TblHdrLoc
   JP    HTML_SaveObj			;setup where table header info is
;=========================================================================
;Name     html_cmd_tfoot
;Purpose  To handle the html command <tfoot....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_ChkTblState
;         TBD
;Notes    1. The table footer HTML command is used primarily for the
;            hard copy output since pages are of fixed length and that
;            tables need to column footers across the pages.
;=========================================================================
html_cmd_tfoot
   CALL  HTM_ChkTblState		;Collecting table info?
   RET   Z					;No
   LD    DE,HTML_TblFtrLoc
   JP    HTML_SaveObj			;setup where table footer info is
;=========================================================================
;Name     html_cmd_tbody
;Purpose  To handle the html command <tbody....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_ChkTblState
;         TBD
;Notes    1. There is no means to perform the TBODY function of
;            independent scrolling of data, so this command simply draws
;            a line across the table to show a new body is encountered.
;=========================================================================
html_cmd_tbody
   CALL  HTM_ChkTblState		;Collecting table info?
   JR    Z,html_cmd_tbody1		;No
   LD    A,(HTML_TblBdyCnt)
   INC   A
   LD    (HTML_TblBdyCnt),A
   RET
html_cmd_tbody1
;
;  Show new body marker on screen
;  reset formats flag for data displayed/row spans
;
   RET
;=========================================================================
;Name     html_cmd_caption
;Purpose  To handle the html command <caption....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         KB_Buffer has HTML Command
;         HTML_CmdTemp = address within KB_Buffer
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblCapLoc = location of table caption
;         HTML_CapLocn = location of caption command data.
;         HTML_CapAlign = alignment of caption.
;         HTML_ParamTyp = corrupted.
;         HTML_ParamLoc = corrupted.
;         HTML_CmdEnd = end of command.
;Uses     TBD
;Call By  TBD
;Calls    HTM_ChkTblState
;         HTM_Get1stCmdChr
;         HTM_GetCmdParam
;         HTM_FindParmLbl
;         HTM_GetCmdParmVal
;         HTM_GetAlignType
;         HTM_SkipCmdObj
;Notes    1. The caption for a table is always centered text to the width
;            of the table render.
;=========================================================================
html_cmd_caption
   CALL  HTM_ChkTblState		;Collecting table info?
   JR    Z,html_cmd_cap3		;No (skip content)
;
   CALL  HTM_Get1stCmdChr		;</caption> command ?
   RET   Z					;Yes
;
   LD    A,HTM_ALIGNTOP
   LD    (HTML_CapAlign),A		;Save default caption is top of table
;
;  Process parameters to command
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_cap2
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    ">"				;end of parameters?
   JR    Z,html_cmd_cap3		;Yes
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_Align
   CP    B					;Is parameter 'align' ?
   JR    NZ,html_cmd_cap2		;No (skip the parameter)
;
   CALL  HTM_GetCmdParmVal		;Get address of parameter value
   CALL  HTM_GetAlignType		;C = which side of table
   LD    A,C
   LD    (HTML_CapAlign),A		;Save where caption is to be shown
;
html_cmd_cap3
   LD    HL,(HTML_CmdEnd)
   LD    A,(HTML_CmdEnd+2)
   LD    (HTML_CapLocn),HL		;Save location for display
   LD    (HTML_CapLocn+2),A
   JP    HTM_SkipCmdObj
;=========================================================================
;Name     html_cmd_col
;Purpose  To handle the html command <col....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         KB_Buffer has HTML Command
;         HTML_CmdTemp = address within KB_Buffer
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. This command is actioned on before the actual display of
;            table data.
;=========================================================================
html_cmd_col
   CALL  HTM_ChkTblState		;Collecting table info?
   RET   Z					;No
   LD    A,1
   LD    (HTML_colgrpspan),A		;min span is 1 column
;=========================================================================
;Name     html_cmd_colgroup
;Purpose  To handle the html command <colgroup....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         KB_Buffer has HTML Command
;         HTML_CmdTemp = address within KB_Buffer
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ParamTyp = corrupted.
;         HTML_ParamLoc = corrupted.
;         HTML_colgrpspan = number of columns the effects spans.
;         HTML_colgrpcc = updated by HTML_colgrpspan
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. This command is actioned on before the actual display of
;            table data.
;=========================================================================
html_cmd_colgroup
   CALL  HTM_ChkTblState		;Collecting table info?
   RET   Z					;No
;
;  Setup the Table format data
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_colgrp2
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    '>'				;no parameter?
   JR    Z,html_cmd_colgrp3		;Yes (adjust tbl fmt data)
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_Align
   CP    B					;Is parameter 'align' ?
   JR    Z,html_cmd_colgrp0a		;Yes
   LD    A,HTM_P_Width
   CP    B					;Is parameter 'width' ?
   JR    Z,html_cmd_colgrp0a		;Yes
   LD    A,HTM_P_Span
   CP    B					;Is parameter 'span' ?
   JR    NZ,html_cmd_colgrp2		;No (skip the parameter)
;
html_cmd_colgrp0a
   LD    (HTML_ParamTyp),A
   CALL  HTM_GetCmdParmVal		;Get address of parameter value
   LD    A,(HTML_ParamTyp)
   CP    HTM_P_Align
   CALL  Z,HTM_SetColGrpAlign		;save Group Align value
   LD    A,(HTML_ParamTyp)
   CP    HTM_P_Width
   CALL  Z,HTM_SetColGrpWidth		;save group width value
   LD    A,(HTML_ParamTyp)
   CP    HTM_P_Span
   CALL  Z,HTM_SetColGrpSpan		;save group span value
   JR    html_cmd_colgrp2		;Do next parameter if any
;
html_cmd_colgrp3
;
;  Update the table format data for the n columns
;
   CALL  HTM_SetCGAlign			;Setup column alignments
   CALL  HTM_SetCGWTyp			;Setup column widths
   CALL  HTM_SetCGWidth			;Setup width values
   LD    A,(HTML_colgrpspan)
   CP    255				;no span in <COLGROUP..>?
   RET   Z					;Yes (assume <COL> follows)
   LD    B,A
   LD    A,(HTML_colgrpcc)
   ADD   A,B
   LD    (HTML_colgrpcc),A		;update next column index
   RET
;=========================================================================
;Name     html_cmd_th
;Purpose  To handle the html command <th....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         HTML_TblCurrCol = current cell column
;         HTML_TblCurrRow = current cell row
;         KB_Buffer has HTML Command
;         HTML_CmdTemp = address within KB_Buffer
;         HTML_TblIndex = index into tables for info gathering.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblCurrCol = updated cell column
;         HTML_TblCurrRow = current cell row
;Uses     TBD
;Call By  HTM_ShowFileObject via interpretation of HTML.
;Calls    HTM_ChkTblState
;         HTM_ShowCellData
;         HTM_CalcCellWidth
;         HTM_TDTH_Params
;         HTM_SetupWidth
;         HTM_CalcTblFrmtCF
;         HTM_NextFrmtCol
;         HTM_CalcTblFrmtCS
;         HTM_SetupTblRowCol
;Notes    1. When a header column spans more than 1 column the total
;            widths of the spanned columns define the header column width.
;=========================================================================
html_cmd_th
   CALL  HTM_ChkTblState		;Collecting table info?
   JP    Z,HTM_ShowCellData		;No, render content.
;
;  Get cell information for rendering
;
   CALL  HTM_TDTH_Params		;No (process any parameters to cmd)
;
   CALL  HTM_CalcCellWidth		;calculate the max cell width.
;
   LD    HL,HTML_TblState
   BIT   4,(HL)				;1st row of columns defined?
   JR    Z,html_cmd_th2			;No
;
   CALL  HTM_ChkTblEnd			;end of known columns
   RET   Z					;Yes (The <TH> is invalid)
;
   CALL  HTM_CalcTblFrmtRS
   LD    A,(HL)
   OR    A					;cell on multi-row column?
   CALL  NZ,HTM_NextFrmtCol		;Yes (use next free column for <TH>)
;
html_cmd_th2
   CALL  HTM_CalcTblFrmtCS
   LD    A,(HL)				;Get number of columns spanned
   CP    1					;Only 1 column?
   CALL  Z,HTM_SetupWidth		;adjust width of this column
;
html_cmd_th1
   CALL  HTM_SetupSpans			;setup column/row spans
   CALL  HTM_SetupTblRowCol		;setup column formats
   JP    HTM_NextFrmtCol		;Goto next column
;=========================================================================
;Name     html_cmd_td
;Purpose  To handle the html command <td....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         KB_Buffer has HTML Command
;         HTML_CmdTemp = address within KB_Buffer
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblCurrCol = updated table column.
;Uses     TBD
;Call By  HTM_ShowFileObject via interpretation of HTML.
;Calls    HTM_ChkTblState
;         HTM_CalcCellWidth
;         HTM_CalcTblFrmtCF
;         HTM_CalcTblFrmtRS
;         HTM_SetupWidth
;         HTM_NextFrmtCol
;Notes    1. Dont expect row/column spans in the <TD> commands.
;=========================================================================
html_cmd_td
   CALL  HTM_ChkTblState		;Collecting table info?
   JP    Z,HTM_ShowCellData		;No, render content.
;
;  Get cell information for rendering
;
   CALL  HTM_CalcCellWidth		;calculate the max cell width.
;
   CALL  HTM_ChkTblEnd			;end of known columns
   RET   Z					;Yes
;
   CALL  HTM_CalcTblFrmtRS		;Get row spans for this column
   LD    A,(HL)
   OR    A					;cell 'Taken' on multi-row column?
   CALL  NZ,HTM_NextFrmtCol		;Yes (find next valid column for <TD>)
;
   CALL  HTM_CalcTblFrmtCF		;get current cell format info
   BIT   3,(HL)				;Fixed width column?
   JR    NZ,html_cmd_td0		;Yes
;
   CALL  HTM_CalcTblFrmtCS
   LD    A,(HL)				;Get number of columns spanned
   CP    1					;Only 1 column?
   CALL  Z,HTM_SetupWidth		;adjust width of this column
;
html_cmd_td0
   JP    HTM_NextFrmtCol
;=========================================================================
;Name     html_cmd_tr
;Purpose  To handle the html command <tr....> to show a new table row.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         KB_Buffer has HTML Command
;         HTML_CmdTemp = address within KB_Buffer
;         HTML_TblCRY = calculated Cell Row
;         HTML_TblFmtDat = array of column formats
;         HTML_TblState = table status/flags
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblCurrCol = corrupted.
;         HTML_TblIndex = 0
;         HTML_TblSRY = Start row
;Uses     TBD
;Call By  TBD
;Calls    HTM_DecRowSpans
;Notes    TBD
;=========================================================================
html_cmd_tr
   CALL  HTM_DecRowSpans		;decrement row spans for all cols.
;
   CALL  HTM_ChkTblState		;Collecting table info?
   JR    Z,html_cmd_tr1			;No (performing rendering)
;
   CALL  HTM_ReInitColSpans		;reset col spans
;
;  Handle processing during table info building.
;
   LD    HL,(HTML_TblFmtDat)
   LD    A,(HL)
   CP    &C0				;Any column defs stored?
   JR    Z,html_cmd_tr0			;No
;
   LD    HL,HTML_TblState
   SET   4,(HL)				;Flag number of columns now known.
   JR    html_cmd_tr0
;
html_cmd_tr1
;
;  Rendering the table after all data have been processed.
;
   CALL  HTM_SetupEndRow
   PUSH  HL
   CALL  HTM_ChkTblBorder		;Table have borders?
   CALL  NZ,HTM_ShowAllBorders	;Yes (show borders)
   POP   HL
   INC   HL					;next render row
   LD    (HTML_TblSRY),HL		;setup new start row to render
;
   CALL  HTM_ReInitColSpans		;reset col spans
;
;  Determine what the next table row to render starts from.
;
   LD    HL,(HTML_TblCurrRow)		;Track table rows.
   INC   HL
   LD    (HTML_TblCurrRow),HL
;
html_cmd_tr0
   LD    DE,0
   LD    (HTML_TblIndex),DE		;restart at first column
   RET