;=========================================================================
;
; Application Routines (HTML rendering commands)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
; 1. The rendering of the current command does not expect that the same
;    command to be recursed. This may lead to an infinite loop if it does.
;
; 2. Where a command changes the font (e.g normal to italic) the original
;    command will have no way of reverting the font back to what it was.
;
;=========================================================================
;Name     HTM_ShowFile
;Purpose  To show the loaded file data from the banks of memory. If the
;         first entry into this routine determines that the file data is
;         not HTML then the rendering will be just ASCII text.
;In Regs  A = first char of loaded data, may be zero if showing HTML as
;             plain text.
;         B = wrap lock flag (0=No wrapping else wrapping text enabled)
;         C = href label to find flag
;             0 = find the href label, 255 = no search required
;            1-2 = searched text refresh requested.
;         DE = Address of href Label text (if any)
;         HL = address of buffer for HTML to use.
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = current state of rendering.
;                            (if 0 then new page displayed requested)
;         HTML_FCurrPos = start RAM address for the rendering if a find
;                         request was initiated.
;         PageWdw_SCol = current start column of CPC screen for rendering
;         PageWdw_SRow = current start row of CPC screen for rendering.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_RCurrPos = current start position for the actual rendering
;                         engine to display the web-page content.
;         Stream_WebPage = window stream for web-page
;         HTML_PreserveFlag = current text preserve flag
;         HTML_LblTempL = length of label text (0=none)
;         HTML_LblTemp = contains label text from hyperlink.
;         WPHLIndex = number of links in the hyperlinks DB.
;         HTML_quoteLevel = quote level for single/double quotes.
;         HTML_LblTemp = text for #tag label from href.
;         HTML_CmdTemp = location of the buffer for HTML command handling.
;         HTML_PgDBIx = RAM Banks pages Index.
;         KB_Buffer = corrupted.
;         Window_FrameV = corrupted on exit.
;Uses     All registers
;Call By  CPC Browser (Start_App in cpcb-main.asm)
;Calls    HTM_FindURLLabel
;         HTM_AdjustFindAddr
;         HTM_DoHTMLCmd
;         HTM_ShowChar
;         HTM_NextHTMBank
;         HTM_GetFrame
;         HTM_GetFrameAdr
;         HTM_FrmeIStrt
;Notes    1. The HTML file data can only be HTML data if on first entry,
;            a HTML command start is encountered first at position 1.
;         2. To display HTML as text register A must be cleared else the
;            routine will interept the value to determine which mode to
;            use.
;         3. Assume the screen web-page window has been cleared prior to
;            entry.
;         4. The first frame is assumed to be in the browser vars
;=========================================================================
HTM_ShowFile
   LD    (HTML_CmdTemp),HL	;Setup text buffer location.
;
   LD    HL,HTML_WrapLocks
   RES   7,(HL)			;Clear wrap text control
   LD    A,B
   OR    A			;Wrapping text at window edge?
   JR    Z,HTM_ShwFile0		;No
   SET   7,(HL)			;Yes (Flag wrapping enabled)
HTM_ShwFile0
;
   LD    A,(HTML_RenderState)
   AND   &07			;clear rendering flags
   LD    (HTML_RenderState),A
;
   XOR   A
   LD    (HTML_WPHLIndex),A	;Reset DB index of hyperlinks
   LD    (HTML_LblTempL),A	;clear #label length
   LD    (HTML_FrmeSets),A
   LD    (HTML_TblState),A	;Flag table info unknown
   LD    (HTML_NumStyles),A	;No styles in DB (at 1st entry)
   LD    (App_DBIndx),A		;clear DB of App Icons
;
   LD    A,C
   LD    (HTML_TextType),A	;Save search label flag for later
   CP    HTML_C_FindTxtNxt	;Find next requested to show new view
   JR    NZ,HTM_ShwFile1	;No
   LD    HL,HTML_FCurrPos+2
   RES   7,(HL)			;Setup found position for adjustment
HTM_ShwFile1
;
   LD    A,(HTML_StyleFlgs)
   CP    &80			;first time into HTML display
   JR    NZ,HTM_ShwFile2
   XOR   A
   LD    (HTML_StyleFlgs),A	;turn flag off
;
   LD    DE,URL_EndAddr
   LD    HL,HTML_CurrLoc
   LD    BC,3
   LDIR				;Save end of HTML for new objects
HTM_ShwFile2
;
   CALL  HTML_NewFrameOffset
   CALL  HTM_Get1stFrame
;
   LD    A,(Window_FrmeID)
HTM_ShwFileLoop
   BIT   3,A			;Frame available to be seen?
   CALL  Z,HTM_ShowFileObject	;Yes, Show frames contents
;
   LD    HL,WindowFrmeStat
   BIT   7,(HL)			;frame display complete?
   JR    NZ,HTM_ShwFileEx1	;Yes
;
;  check if frames were used (for plain text files frames are
;  never used, so check if the file is non-html is not required)
;
   LD    A,(HTML_FrameFlg)
   BIT   7,A			;Were frames Used?
   JR    Z,HTM_ShwFileEx1	;No
   BIT   5,A			;Are frame objects to be loaded?
   JR    Z,HTM_ShwFileEx	;No
   RES   5,A
   LD    (HTML_FrameFlg),A	;Clear flag for next entry
   CALL  HTM_LoadFrameObjs	;Yes (load all the frame's objects)
   LD    A,1
   LD    (HTML_FrameCurrID),A	;Start at 1st user frame on exit
   CALL  HTM_FrameTaken		;flag primary frame now hidden.
;
   LD    A,(Stream_WebPage)
   CALL  SC_SEL_STREAM		;select web page
   CALL  TXT_CLR_WIN		;clear web-page
;
;  Search for #label after frames were loaded (if required)
;
   LD    A,(URL_LblType)
   CP    HTML_C_FindURL		;Need to find #label?
   CALL  Z,HTM_FindURLHlink	;Yes, Search for Label from hyperlink
; 
HTM_ShwFileEx
   LD    A,(HTML_FrameCurrID)
   INC   A
   LD    (HTML_FrameCurrID),A
   CALL  HTM_GetFrame		;Window_FrameV = next frame
   BIT   0,A			;At end of the Frames DB?
   JR    NZ,HTM_ShwFileLoop	;No
HTM_ShwFileEx1
   LD    HL,WindowFrmeStat
   RES   7,(HL)
   LD    HL,CPCB_Timer
   SET   6,(HL)
   CALL  CPCB_ShowTimer
   RET
